package org.jharks.workflow.survey.engine.cache;

import org.jharks.workflow.survey.engine.model.QuestionGroupConfiguration;


/**
 * Provides a cache of {@link QuestionGroupConfiguration} objects.
 *
 * <P>
 * Implementations should provide appropriate methods to set their cache
 * parameters (eg time-to-live) and/or force removal of entities before their
 * normal expiration. These are not part of the <code>DynamicQuestionGroupCache</code>
 * interface contract because they vary depending on the type of caching
 * system used (eg in-memory vs disk vs cluster vs hybrid).
 * </p>
 *
 * @author Michael J. Rizzo
 */
public interface DynamicQuestionGroupCache {

    /**
     * Obtains a <objec>QuestionGroupConfiguration</object> from the cache.
     *
     * @param questionGroup used to place the configuration object in the cache
     *
     * @return the populated <code>QuestionGroupConfiguration</code> or 
     *         <code>null</code> if the configuration object could not be found 
     *         or if the cache entry has expired
     */
    public QuestionGroupConfiguration getConfigurationFromCache(String questionGroup);
    
    /**
     * Places a <object>QuestionGroupConfiguration</object> in the cache. 
     * The questiongroup is the key used to subsequently
     * retrieve the <code>QuestionGroupConfiguration</code>.
     *
     * @param user the fully populated <code>QuestionGroupConfiguration</code> to place in the cache
     */
    public void putConfigurationInCache(QuestionGroupConfiguration configuration);
    
    /**
     * Removes the specified configuration from the cache. The 
     * <code>questionGroup</code> is the key used to remove the configuration.
     * If the configuration is not found, the method should simply return 
     * (not throw an exception).<P>Some cache implementations may not support 
     * eviction from the cache,  in which case they should provide appropriate
     * behaviour to alter the configuration in either its documentation, 
     * via an exception, or through a log message.</p>
     *
     * @param questionGroup representing the configuration object to be 
     *        evicted from the cache.
     */
    public void removeConfigurationFromCache(String questionGroup);
    
}
