package org.jharks.workflow.survey.engine.cache;

import org.jharks.workflow.survey.util.CustomDbResourceBundle;


/**
 * Provides a cache of {@link org.jharks.workflow.survey.util.CustomDbResourceBundle} objects.
 * 
 * <p>
 * Implementations should provide appropriate methods to set their cache
 * parameters (eg time-to-live) and/or force removal of entities before their
 * normal expiration. These are not part of the <code>WorkFlowResourceBundleCache</code>
 * interface contract because they vary depending on the type of caching
 * system used (eg in-memory vs disk vs cluster vs hybrid).
 * </p>
 * 
 * @author Michael J. Rizzo
 *
 */
public interface WorkFlowResourceBundleCache {

	/**
	 * Obtains a <object>CustomDbResourceBundle</object> from the cache.
	 * 
	 * @param bundleKey the key that was used to place the bundle in the cache.
	 * @return the populated <object>CustomDbResourceBundle</object> or <code>null</code> 
	 *         if the CustomDbResourceBundle could not be found or recreated from the database.
	 */
	public CustomDbResourceBundle getBundleFromCache(WorkFlowResourceBundleCacheKey bundleKey);
	
	
	/**
	 * Palces a <object>CustomDbResourceBundle</object> in the cache.
	 * The <object>WorkFlowResourceBundleCacheKey</object> is the key used to 
	 * subsequently retrieve the <object>CustomDbResourceBundle</object>
	 * 
	 * @param bundle the fully populated CustomDbResourceBundle to place in the cache.  
	 */
	public void putBundleInCache(CustomDbResourceBundle bundle);
	
	/**
	 * Removes the specified bundle from the cache.  The <code>WorkFlowResourceBundleCacheKey</code>
	 * is the key used to remove the bundle.  If the bundle is not found, the method should
	 * simply return (not throw an exception. <P>Some cache implementations may not support 
     * eviction from the cache,  in which case they should provide appropriate
     * behaviour to alter the configuration in either its documentation, 
     * via an exception, or through a log message.</p>
	 * @param bundleKey the key that was used to place the bundle in the cache.
	 */
	public void removeBundleFromCache(CustomDbResourceBundle bundle);
}
