package org.jharks.workflow.survey.engine.dao;

import java.io.IOException;
import java.io.InputStream;


/**
 * Interface that defines methods that need to be implemented by a workflow definition data access object.
 * The interface defines methods for persisting xml workflow definition files and for retrieving definitions 
 * to be passed to the workflow engine for instantiation of workflows.
 * 
 * @author Michael J. Rizzo
 *
 */
public interface XmlWorkflowDefinitionDao {

	/**
	 * Persist the xml definition content that is contained within the stream to the 
	 * xml definition repository.
	 * @param definitionName the name to use for the definition.
	 * @param version the version number for this definition to be used for this persistent tx.
	 * @param definitionContentStream the stream of definition content to persist.
 	 */
	public void persistXmlDefinition(String definitionName, Long version, InputStream definitionContentStream);
	
	/**
	 * Retrieve the xml definition as a stream.
	 * @param definitionId the id of the definition to retrieve.
	 * @return the input stream containing the definition content.
	 */
	public InputStream getXmlDefinitionAsStream(Long definitionId);
	
	/**
	 * Retrieve the xml definition as a stream for an existing workflow definition entry.  Automatically
	 * determine the latest version of the workflow definition stored within the persistent
	 * repository and returns the workflow definition content as a stream.
	 * @param definitionName the name for the definition.
	 * @return the xml content of the workflow definition as a stream.
	 */
	public InputStream getXmlDefinitionLatestVersionAsStream(String definitionName);
	
	/**
	 * Return the xml definition content as a string.
	 * @param definitionId the id of the definition to retrieve.
	 * @return the xml definition as string.
	 */
	public String getXmlDefinitionAsString(Long definitionId);
	
	/**
	 * Retrieve the xml definition as a string for an existing workflow definition entry.  Automatically
	 * determine the latest version of the workflow definition stored within the persistent
	 * repository and returns the workflow definition content as a string.
	 * @param definitionName the name for the definition.
	 * @return the xml content of the workflow definition as a String.
	 */
	public String getXmlDefinitionLatestVersionAsString(String definitionName);
	
	
	/**
	 * Retrieve the definition id for an existing workflow definition entry.
	 * @param definitionName the name for the definition.
	 * @param definitionVersion the version of the definition.
	 * @return the id for the definition if found.
	 */
	public Long getDefinitionId(String definitionName, Long definitionVersion);
	
	/**
	 * Retrieve the definition id for an existing workflow definition entry.  Automatically
	 * determine the latest version of the workflow definition stored within the persistent
	 * repository and return the id of the latest version.
	 * @param definitionName the name for the definition.
	 * @return the id for the latest version of the passed definition name.
	 */
	public Long getDefinitionIdLatestVersion(String definitionName);
	
	
	/**
     * Helper method that converts an <code>InputStream</code> to a String.
     * @param is the input stream to convert.
     * @return the converted string
     * @throws IOException if error occurs processing the input stream.
     */
    public String convertStreamToString(InputStream is) throws IOException;
}
