package org.jharks.workflow.survey.engine.dao.impl;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ResourceBundle;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.springframework.jdbc.core.JdbcTemplate;

/**
 * Abstract JDBC Dao class containing common methods for JDBC 
 * Dao implementations that are not leveraging the spring 
 * JDBC template.
 * 
 * @author Michael J. Rizzo
 *
 */
public abstract class AbstractJdbcDaoImpl {
	
	/** The Logger instance for this class. */
	private static Logger LOGGER = Logger.getLogger(AbstractJdbcDaoImpl.class);
	
	/** The set Database Scema for the application install. */
	protected static final String DATABASE_SCHEMA = getSchemaValue();
	
	/** The injected datasource. */
    private DataSource dataSource; 
    
    /** The jdbc template for this instance. */
    private JdbcTemplate jdbcTemplate;
    
    /**
     * Sets the Dao's datasource.
     * @param dataSource The datasource to be set.
     */
    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
        this.jdbcTemplate = new JdbcTemplate(dataSource);
    }
    
    /**
     * Returns the injected datasource.
     * @return the datasource.
     */
    public DataSource getDataSource() {
        return this.dataSource;
    }
    
    /**
     * Return the associated jdbc template.
     * @return the jdbc template.
     */
    public JdbcTemplate getJdbcTemplate() {
    	return this.jdbcTemplate;
    }
    
    /**
     * Retrieves a database connection from the injected 
     * <code>DataSource</code> with auto commit set to false.
     * @return the database connection.
     * @throws SQLException if error occurs retrieving connection.
     */
    public Connection getConnection() throws SQLException {
        return getConnection(false);
    }
    
    /**
     * Retrieves a database connection from the injected 
     * <code>DataSource</code> setting auto commit property of the 
     * connection to the passed value.
     * @param autoCommit The boolean value to which to set the autoCommit
     *        property of the jdbc database connection.
     * @return the database connection.
     * @throws SQLException if error occurs retrieving connection.
     * @throws SQLException
     */
    public Connection getConnection(boolean autoCommit) throws SQLException  {
        Connection conn = dataSource.getConnection();
        conn.setAutoCommit(autoCommit);
        return conn;
    }
    
    /**
     * Performs the search specified by this search criteria, returning the results.
     *
     * @param conn the database connection to use.
     * @param numResults the fetch size for the SQL query.
     *  @param sqlQuery the sql query to execute.
     *
     * @return a ResultSet containing the search results.
     *
     * @exception SQLException thrown if there is an error while retrieving the results.
     */
    public ResultSet getResultSet(Connection conn, int numResults, String sqlQuery) throws SQLException {
        
       Statement stmt = conn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, 
               ResultSet.CONCUR_READ_ONLY);
       stmt.setFetchSize(numResults + 1);
       return stmt.executeQuery(sqlQuery);
    }
 
    private static String getSchemaValue() {
    	ResourceBundle bundle = 
            ResourceBundle.getBundle("JharksWorkflowSurveyResources");
    	String databaseSchema = bundle.getString("database.schema");
    	String schemaValue = "";
    	if (databaseSchema != null && !databaseSchema.trim().equals("")) {
    		LOGGER.debug("database.schema resource key = [ " + databaseSchema.trim() + " ]");
    		schemaValue = databaseSchema + "."; 
    	}
    	LOGGER.info("SCHEMA VALUE FOR APPLICATION INSTANCE SET TO = " + schemaValue);
    	return schemaValue; 	
    }
}
