package org.jharks.workflow.survey.engine.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.jharks.workflow.survey.engine.dao.QuestionDao;
import org.jharks.workflow.survey.engine.model.AnswerChoice;
import org.jharks.workflow.survey.engine.model.Question;
import org.jharks.workflow.survey.engine.model.QuestionDefinition;
import org.jharks.workflow.survey.engine.model.QuestionGroup;
import org.jharks.workflow.survey.engine.model.QuestionGroupConfiguration;
import org.jharks.workflow.survey.util.DatabaseUtilities;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.dao.DataRetrievalFailureException;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;


/**
 * A question dao implementation that leverages Spring IoC injection.
 * 
 * @author Michael J. Rizzo
 */
public class QuestionJdbcDaoImpl extends AbstractJdbcDaoImpl implements QuestionDao {

    /** The logger for this class. */
    private static final Logger logger = Logger.getLogger(QuestionJdbcDaoImpl.class);
    
    /** SQL string used for question retrieval. */
    private static final String QUESTION_RETRIEVAL = "SELECT QUESTION_ID, " +
            "QUESTION_KEY FROM " + DATABASE_SCHEMA + "QUESTION WHERE CURRENT_FLAG = 1";
    
    /** SQL string used to retrieve question count for a group */
    private static final String QUESTION_COUNT_BY_GROUP = "SELECT " +
            "QG.QUESTION_GROUP, COUNT(Q.QUESTION_ID) AS QUESTION_COUNT FROM "
            + DATABASE_SCHEMA + "QUESTION Q, "
            + DATABASE_SCHEMA + "QUESTION_GROUP QG, "
            + DATABASE_SCHEMA + "QUESTION_GROUP_QUESTION QGQ " +
            "WHERE Q.QUESTION_ID = " +
            "QGQ.QUESTION_ID AND QG.QUESTION_GROUP_ID = QGQ.QUESTION_GROUP_ID AND QG.QUESTION_GROUP = ? " +
            "GROUP BY QG.QUESTION_GROUP";
    
    /** SQL string used to retrieve answer choices for question group. */
    private static final String QUESTION_ANSCHOICE_RET_BY_GROUP = 
        "SELECT QG.QUESTION_GROUP AS QUESTION_GROUP, Q.SAVE_ANSWER " +
        "AS SAVE_ANSWER, Q.QUESTION_ID AS QUESTION_ID, " +
        "Q.QUESTION_KEY AS QUESTION_KEY, QL1.LAYOUT_NAME AS LAYOUT_NAME, " + 
        "AC.ANSWER_CHOICE_ID AS ANSWER_CHOICE_ID, " +
        "AC.ANSWER_CHOICE_ALIAS_ID AS ANSWER_CHOICE_ALIAS_ID, " +
        "AC.ANSWER_CHOICE_LABEL AS ANSWER_CHOICE_LABEL, " +
        "AC.ANSWER_KEY AS ANSWER_KEY, AC.EXPLANATION " +
        "AS ANSWER_EXPLANATION, QAC.SUBQUESTION_GROUP_ID AS SUBQUESTION_GROUP_ID " + 
        "FROM " 
        + DATABASE_SCHEMA + "QUESTION Q " + 
        "LEFT JOIN " + DATABASE_SCHEMA + "QUESTION_LAYOUT QL1 ON Q.QUESTION_LAYOUT_ID = QL1.LAYOUT_ID " +
        "LEFT JOIN " + DATABASE_SCHEMA + "QUESTION_ANSWER_CHOICE QAC ON Q.QUESTION_ID = QAC.QUESTION_ID " + 
        "LEFT JOIN " + DATABASE_SCHEMA + "ANSWER_CHOICE AC ON QAC.ANSWER_CHOICE_ID = AC.ANSWER_CHOICE_ID "  +
        "INNER JOIN " + DATABASE_SCHEMA + "QUESTION_GROUP_QUESTION QGQ ON Q.QUESTION_ID = QGQ.QUESTION_ID " +
        "INNER JOIN " + DATABASE_SCHEMA + "QUESTION_GROUP QG ON QGQ.QUESTION_GROUP_ID = QG.QUESTION_GROUP_ID " + 
        "WHERE " +
        "QG.QUESTION_GROUP = ? AND Q.CURRENT_FLAG = 1 " +
        "ORDER BY QGQ.QUESTION_SEQUENCE, QAC.CHOICE_SEQUENCE";
    
    /** 
     * SQL string used to retrieve answer choices for question group where the question group
     * is specified by it's unique identifier from the database . 
     */
    private static final String QUESTION_ANSCHOICE_RET_BY_GROUP_ID = 
        "SELECT QG.QUESTION_GROUP AS QUESTION_GROUP, Q.SAVE_ANSWER " +
        "AS SAVE_ANSWER, Q.QUESTION_ID AS QUESTION_ID, " +
        "Q.QUESTION_KEY AS QUESTION_KEY, QL1.LAYOUT_NAME AS LAYOUT_NAME, " + 
        "AC.ANSWER_CHOICE_ID AS ANSWER_CHOICE_ID, " +
        "AC.ANSWER_CHOICE_ALIAS_ID AS ANSWER_CHOICE_ALIAS_ID, " +
        "AC.ANSWER_CHOICE_LABEL AS ANSWER_CHOICE_LABEL, " +
        "AC.ANSWER_KEY AS ANSWER_KEY, AC.EXPLANATION " +
        "AS ANSWER_EXPLANATION, QAC.SUBQUESTION_GROUP_ID AS SUBQUESTION_GROUP_ID " + 
        "FROM " 
        + DATABASE_SCHEMA + "QUESTION Q " + 
        "LEFT JOIN " + DATABASE_SCHEMA + "QUESTION_LAYOUT QL1 ON Q.QUESTION_LAYOUT_ID = QL1.LAYOUT_ID " +
        "LEFT JOIN " + DATABASE_SCHEMA + "QUESTION_ANSWER_CHOICE QAC ON Q.QUESTION_ID = QAC.QUESTION_ID " + 
        "LEFT JOIN " + DATABASE_SCHEMA + "ANSWER_CHOICE AC ON QAC.ANSWER_CHOICE_ID = AC.ANSWER_CHOICE_ID, "  
        + DATABASE_SCHEMA + "QUESTION_GROUP QG, " 
        + DATABASE_SCHEMA + "QUESTION_GROUP_QUESTION QGQ " +
        "WHERE Q.QUESTION_ID = QGQ.QUESTION_ID AND QGQ.QUESTION_GROUP_ID = QG.QUESTION_GROUP_ID " +
        "AND QG.QUESTION_GROUP_ID = ? AND Q.CURRENT_FLAG = 1 " +
        "ORDER BY QGQ.QUESTION_SEQUENCE, QAC.CHOICE_SEQUENCE";
    
    
    
    /** The question map populated from question table configuration. */
    private Map<String, Long> questionMap = new HashMap<String, Long>(); 
    
    /**
     * {@inheritDoc}
     */
    @SuppressWarnings("unchecked")
	public void populateQuestionIds() {
        
        List<QuestionDefinition> questionDefinitions = this.getJdbcTemplate().query(QUESTION_RETRIEVAL, new QuestionDefinitionRowMapper());
            
            for (QuestionDefinition questionDefinition : questionDefinitions) {
				questionMap.put(questionDefinition.getQuestionKey(), questionDefinition.getQuestionId());
			}
    }
    
    /**
	 * Inner class to map a question definition query result row to a question definition object.
	 */
	private static final class QuestionDefinitionRowMapper implements ParameterizedRowMapper<QuestionDefinition> {
		
		/**
		 * Maps the result row to the dictation object.
		 */
		public QuestionDefinition mapRow(ResultSet rs, int rowNum) throws SQLException {
			QuestionDefinition questionDef = new QuestionDefinition(rs.getString("QUESTION_KEY"),
					rs.getLong("QUESTION_ID"));
			
			return questionDef;
		}	
		
	}
	

    
    /**
     * {@inheritDoc}
     */
    public Collection<Question> getQuestionsByGroup(String groupId) {
        Connection conn = null;
        
        try {
            conn = getConnection(false);
            return getQuestionsByGroup(groupId, conn);
        } catch (SQLException sqle) {
            logger.error("Error occurred retrieving questions", sqle);
            throw new DataIntegrityViolationException(sqle.getMessage(), sqle);
        } finally {
            DatabaseUtilities.cleanup(conn);
        }
    }
    
    /**
     * Returns the set of questions that are mapped to the passed group id.  The
     * group represents any form of page grouping such as a page.
     * @param groupId the group id for which to retrieve mapped questions.
     * @paran conn a pre-fetched database connection.
     * @return the set of questions that are contained within the passed group 
     *         id.
     */
    protected Collection<Question> getQuestionsByGroup(String groupId, Connection conn) {
        PreparedStatement statement = null;
        ResultSet rs = null;
        try { 
          statement = conn.prepareStatement(QUESTION_ANSCHOICE_RET_BY_GROUP);
        
          statement.setString(1,groupId);
        
          if (logger.isDebugEnabled())
            logger.debug("SQL QUERY = [ " + statement.toString() + "]");
        
          rs = statement.executeQuery();
        
          List<Question> questions = new LinkedList<Question>();
          Map<Long, Question> questionsMap = new HashMap<Long, Question>();
          while (rs.next()) {
             Question question = null;
             Long questionId = new Long(rs.getLong("QUESTION_ID"));
             if (questionsMap.containsKey(questionId)) {
                question = (Question) questionsMap.get(questionId);
             } else {
                question = readQuestionResult(rs);
                questionsMap.put(questionId, question);
                questions.add(question);
             }
           
             AnswerChoice choice = readAnswerChoice(rs, conn);
             if (choice != null) {
                question.addAnswerChoice(choice);
             }
          }
        
        return questions;
        
        } catch (SQLException sqle) {
          logger.error("Error occurred retrieving questions", sqle);
          throw new DataIntegrityViolationException(sqle.getMessage(), sqle);
        } finally{
          DatabaseUtilities.cleanup(rs, statement);
        }
    }
    
    /**
     * Returns the set of questions that are mapped to the passed group id.  The
     * group represents any form of page grouping such as a page.
     * @param groupId the unique numeric group id for which to retrieve mapped questions.
     * @param questionGroup the passed questionGroup on which the name will be set.
     * @paran conn a pre-fetched database connection.
     * @return the set of questions that are contained within the passed group 
     *         id.
     */
    protected Collection<Question> getQuestionsByGroupId(Long groupId, QuestionGroup questionGroup, Connection conn) {
        PreparedStatement statement = null;
        ResultSet rs = null;
        try { 
          statement = conn.prepareStatement(QUESTION_ANSCHOICE_RET_BY_GROUP_ID);
        
          statement.setLong(1,groupId);
        
          if (logger.isDebugEnabled())
            logger.debug("SQL QUERY = [ " + statement.toString() + "]");
        
          rs = statement.executeQuery();
        
          List<Question> questions = new LinkedList<Question>();
          Map<Long, Question> questionsMap = new HashMap<Long, Question>();
          while (rs.next()) {
             Question question = null;
             Long questionId = new Long(rs.getLong("QUESTION_ID"));
             String questionGroupNameId = rs.getString("QUESTION_GROUP");
             questionGroup.setGroupId(questionGroupNameId);
             if (questionsMap.containsKey(questionId)) {
                question = (Question) questionsMap.get(questionId);
             } else {
                question = readQuestionResult(rs);
                questionsMap.put(questionId, question);
                questions.add(question);
             }
           
             AnswerChoice choice = readAnswerChoice(rs, conn);
             if (choice != null) {
                question.addAnswerChoice(choice);
             }
          }
        
        return questions;
        
        } catch (SQLException sqle) {
          logger.error("Error occurred retrieving questions", sqle);
          throw new DataIntegrityViolationException(sqle.getMessage(), sqle);
        } finally{
          DatabaseUtilities.cleanup(rs, statement);
        }
    }
    
    /**
     * {@inheritDoc}
     */
    public QuestionGroupConfiguration getQuestionGroupConfiguration(String groupId) {
        
        Connection conn = null;
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        QuestionGroupConfiguration config = null;
        try {
            conn = getConnection(false);
            pstmt = conn.prepareStatement(QUESTION_COUNT_BY_GROUP);
            pstmt.setString(1, groupId);
            rs = pstmt.executeQuery();
            
            if (rs.next())
                config = readCountResult(rs);
            
            return config;
        } catch (SQLException sqle) {
            logger.error("Error occurred retrieving question count " +
                    "for specified group", sqle);
            throw new DataRetrievalFailureException("Error occurred retrieving question count " +
                    "for specified group", sqle);
        } finally {
            DatabaseUtilities.cleanup(conn, rs, pstmt);
        }
        
    }
    
    /**
     * Reads the current row of a result set into an instance of 
     * <CODE>Question</CODE>.
     * 
     * @param rs the result set to read.
     * 
     * @return a <CODE>Question</CODE> instance retrieved from the result set.
     * 
     * @throws SQLException thrown if there is an error while reading the result set.
     * 
     */
    protected Question readQuestionResult(ResultSet rs) throws SQLException {
        Question question = new Question();
        
        question.setQuestionId(rs.getLong("QUESTION_ID"));
        question.setQuestionKey(rs.getString("QUESTION_KEY"));
        question.setLayoutName(rs.getString("LAYOUT_NAME"));
        question.setGroupId(rs.getString("QUESTION_GROUP"));
        
        int saveAnswer = rs.getInt("SAVE_ANSWER");
        question.setSaveAnswer(saveAnswer == 1);
        
        return question;
    }
    
    /**
     * Reads the current row of the result set into an instance of 
     * <CODE>QuestionGroupConfiguration</CODE>
     * @param rs the result set from which to read a record.
     * @return the newly created <code>QuestionGroupConfiguration</code> 
     *         instance.
     * @throws SQLException if error occurs reading question count result set.
     */
    protected QuestionGroupConfiguration readCountResult(ResultSet rs) 
      throws SQLException {
        QuestionGroupConfiguration config = new QuestionGroupConfiguration();
        config.setNumberOfQuestion(rs.getInt("QUESTION_COUNT"));
        config.setQuestionGroupName(rs.getString("QUESTION_GROUP"));
        return config;
    }
    
    /**
     * Reads the current row of the result set into an instance of 
     * <CODE>AnswerChoice</CODE>.
     * 
     * @param rs the result set to read.
     * 
     * @return an instance of <CODE>AnswerChoice</CODE> populated from the 
     *         current row of the result set.
     * 
     * @throws SQLException thrown if there is an error while reading 
     *         the result set.
     * @throws CHException thrown if there is an error while reading 
     *         the subquestions for the result set.
     */
    protected AnswerChoice readAnswerChoice(ResultSet rs, Connection conn) 
      throws SQLException {
       String answerKey = rs.getString("ANSWER_KEY");
       if (answerKey == null) {
          return null;
       } else {
          AnswerChoice ac = new AnswerChoice();
          ac.setAnswerKey(answerKey);
          ac.setId(rs.getLong("ANSWER_CHOICE_ID"));
          ac.setAnswerChoiceAliasId(rs.getString("ANSWER_CHOICE_ALIAS_ID"));
          ac.setAnswerLabel(rs.getString("ANSWER_CHOICE_LABEL"));
          ac.setExplanation(rs.getString("ANSWER_EXPLANATION"));
          
          long subQuestionGroupId = rs.getLong("SUBQUESTION_GROUP_ID");
          if (subQuestionGroupId > 0) {
             QuestionGroup subQuestions = new QuestionGroup(subQuestionGroupId);
             Collection<Question> questions = getQuestionsByGroupId(subQuestionGroupId, subQuestions, conn);
             subQuestions.addAll(questions);
             
             ac.setSubQuestions(subQuestions);
          }
          
          return ac;
       }
    }    
    
    /**
     * {@inheritDoc}
     */
    public Long getQuestionId(String questionKey) {
        return questionMap.get(questionKey);
    }
    
    /**
     * Method for retrieval of pre-cached question ids by <code>
     * PersistenceManager</code>.
     * @param questionKey the question key to look up.
     * @return the question id or null if a question is not found for the passed
     *         key.
     */
    public Long getLoadedQuestionId(String questionKey) {
        return questionMap.get(questionKey);
    }
}
