package org.jharks.workflow.survey.engine.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;
import java.util.TreeMap;

import org.apache.log4j.Logger;
import org.jharks.workflow.Answer;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.survey.engine.dao.QuestionDao;
import org.jharks.workflow.survey.engine.dao.ResponseDao;
import org.jharks.workflow.survey.engine.dto.QuestionAnswerSubquestionPairDto;
import org.jharks.workflow.survey.engine.model.AnswerChoice;
import org.jharks.workflow.survey.engine.model.GroupAnswers;
import org.jharks.workflow.survey.engine.model.GroupAnswersKey;
import org.jharks.workflow.survey.engine.model.GroupBasedAnswerSet;
import org.jharks.workflow.survey.engine.model.Question;
import org.jharks.workflow.survey.engine.model.QuestionAnswers;
import org.jharks.workflow.survey.engine.model.QuestionGroup;
import org.jharks.workflow.survey.engine.model.SurveyAnswer;
import org.jharks.workflow.survey.util.DatabaseUtilities;
import org.jharks.workflow.survey.util.ResourceUtilities;
import org.springframework.dao.DataRetrievalFailureException;



public class ResponseJdbcDaoImpl extends AbstractJdbcDaoImpl implements ResponseDao {
    
    private static Logger logger = Logger.getLogger(ResponseJdbcDaoImpl.class);
    
    /** The injected or set question dao instance. */
    private QuestionDao questionDao;
    
    private static final String RESPONSES_SUM_BASE = "SELECT SRS.RESPONSE_SET_ID, QG.QUESTION_GROUP_ID, QG.QUESTION_GROUP, " +
            "QGQ.QUESTION_SEQUENCE, Q.QUESTION_KEY, Q.QUESTION_ID, AC.ANSWER_CHOICE_ID, " +
            "QAC.CHOICE_SEQUENCE, A.SURVEY_ANSWER_ID, A.ANSWER_KEY, QAC.SUBQUESTION_GROUP_ID, " +
            "SQRS.RESPONSE_SET_ID AS \"SUBQUESTION_RESPONSE_SET_ID\" " +
            "FROM " 
            + DATABASE_SCHEMA + "SURVEY_ANSWER A " +
            "INNER JOIN " + DATABASE_SCHEMA + "SURVEY_RESPONSE_SET SRS ON (SRS.RESPONSE_SET_ID = A.RESPONSE_SET_ID) " +
            "INNER JOIN " + DATABASE_SCHEMA + "SURVEY S ON (S.SURVEY_ID = SRS.SURVEY_ID) " +
            "INNER JOIN " + DATABASE_SCHEMA + "QUESTION Q ON (Q.QUESTION_ID = SRS.QUESTION_ID) " +
            "INNER JOIN " + DATABASE_SCHEMA + "QUESTION_GROUP_QUESTION QGQ ON (QGQ.QUESTION_ID = Q.QUESTION_ID) " +
            "INNER JOIN " + DATABASE_SCHEMA + "QUESTION_GROUP QG ON (QG.QUESTION_GROUP_ID = QGQ.QUESTION_GROUP_ID) " +
            "LEFT OUTER JOIN " + DATABASE_SCHEMA + "ANSWER_CHOICE AC ON (A.ANSWER_KEY = AC.ANSWER_KEY) " +
            "LEFT OUTER JOIN " + DATABASE_SCHEMA + "QUESTION_ANSWER_CHOICE QAC ON (AC.ANSWER_CHOICE_ID = QAC.ANSWER_CHOICE_ID AND QAC.QUESTION_ID = Q.QUESTION_ID) " +
            "LEFT OUTER JOIN " + DATABASE_SCHEMA + "QUESTION_GROUP_QUESTION SQQG ON (QAC.SUBQUESTION_GROUP_ID = SQQG.QUESTION_GROUP_ID) " +
            "LEFT OUTER JOIN " + DATABASE_SCHEMA + "SURVEY_RESPONSE_SET SQRS ON (SQQG.QUESTION_ID = SQRS.QUESTION_ID AND S.SURVEY_ID = SQRS.SURVEY_ID) " +
            "WHERE ";
    		
    		
    private static final String RESPONSES_SUM_BY_GROUP = RESPONSES_SUM_BASE +
    		"S.SURVEY_ID = ? ORDER BY SRS.RESPONSE_SET_ID ,QG.QUESTION_GROUP, QGQ.QUESTION_SEQUENCE, CHOICE_SEQUENCE";
    
    private static final String RESPONSES_BY_PAGE_GROUP =  "SELECT * FROM (" + RESPONSES_SUM_BASE +
    		"S.SURVEY_ID = ? ) START WITH QUESTION_GROUP = ? CONNECT BY PRIOR SUBQUESTION_GROUP_ID = QUESTION_GROUP_ID " +  
    		//"AND S.SURVEY_ID = ? ) START WITH QUESTION_GROUP = ? CONNECT BY PRIOR SUBQUESTION_GROUP_ID = QUESTION_GROUP_ID " +  
            "ORDER BY RESPONSE_SET_ID ,QUESTION_GROUP, QUESTION_SEQUENCE, CHOICE_SEQUENCE";
    
    /**
     * Initializes the dao implementation and populates the question and answer
     * code maps.
     */
    public void init() {
    	//currently no actions needed.
    }
    
    
    /**
     * {@inheritDoc}
     */
    public GroupBasedAnswerSet getResponsesSummary(Long surveyId) {
       return getResponsesSummary(surveyId, false, null);
    }
    
    /**
     * {@inheritDoc}
     */
    public GroupBasedAnswerSet getResponsesSummary(Long surveyId, boolean localizeAnswerGroups, ResourceBundle bundle) {
        Connection conn = null;
        PreparedStatement statement = null;
        ResultSet rs = null;
        
        GroupBasedAnswerSet answerSet = new GroupBasedAnswerSet();
        
        try {
            conn = getConnection(false);
            statement = conn.prepareStatement(RESPONSES_SUM_BY_GROUP);
            
            if (logger.isDebugEnabled())
              logger.debug("SQL QUERY = [ " + statement.toString() + "]");
            statement.setLong(1, surveyId);
            
            rs = statement.executeQuery();
            
            //if (rs.next()) {
            readResponseSerialResult(rs, answerSet, localizeAnswerGroups, bundle);
            //}
            
            //conn.commit();
            
        } catch (SQLException sqle) {
            logger.error("Error occurred getting responses and mapping to " +
                    "grouped answer set.", sqle);
            throw new DataRetrievalFailureException(sqle.getMessage(), sqle);
        }finally{
            DatabaseUtilities.cleanup(conn, rs, statement);
        }
        
        return answerSet;
    }
    
    
    /**
     * {@inheritDoc}
     */
    public GroupAnswers getResponsesForPageGroup(String pageGroup, SessionHistory sessionHistorySnapshot) {
    	GroupAnswers groupAnswers = new GroupAnswers(pageGroup);
    	
    	
    	Collection<Question> questionsInPageGroup = questionDao.getQuestionsByGroup(pageGroup);
    	for (Question question : questionsInPageGroup) {
			
    		//If the user performed a refresh it is possible that there isn't a snapshot present yet
    		//if sessionHistory is null then skip retrieving answers.
    		if (sessionHistorySnapshot != null) {
    		  List<Answer> answers = sessionHistorySnapshot.getResponses(question.getQuestionKey(), question.getGroupId(), false);
    		
    		  QuestionAnswers questionAnswers = new QuestionAnswers(question);
    		  for (Object object : answers) {
    			String value = ((Answer) object).getValue().toString();
    			SurveyAnswer answer = new SurveyAnswer(value);
    			
				AnswerChoice answerChoice = question.getAnswerChoiceByValue(value);
				
				if (answerChoice != null) {
				  answer.setAnswerChoice(answerChoice);
				  QuestionGroup qg = answerChoice.getSubQuestions();
				  if (qg != null && qg.size() > 0) {
					answer.setSubAnswers(getResponsesForSubquestionPageGroup(pageGroup, qg.getGroupId(), sessionHistorySnapshot));  
				  }
				}
				questionAnswers.add(answer);
			  }
    		  groupAnswers.add(questionAnswers);
    		}
		}
      return groupAnswers;	
    }
    
    /**
     * {@inheritDoc}
     */
    public GroupAnswers getResponsesForSubquestionPageGroup(String parentGroup, String pageGroup, SessionHistory sessionHistorySnapshot) {
    	GroupAnswers groupAnswers = new GroupAnswers(pageGroup);
    		
    	Collection<Question> questionsInPageGroup = questionDao.getQuestionsByGroup(pageGroup);
    	for (Question question : questionsInPageGroup) {
			
    		//If the user performed a refresh it is possible that there isn't a snapshot present yet
    		//if sessionHistory is null then skip retrieving answers.
    		if (sessionHistorySnapshot != null) {
    		  List<Answer> answers = sessionHistorySnapshot.getResponses(question.getQuestionKey(), parentGroup, false);
    		
    		  QuestionAnswers questionAnswers = new QuestionAnswers(question);
    		  for (Object object : answers) {
    			String value = ((Answer) object).getValue().toString();
    			SurveyAnswer answer = new SurveyAnswer(value);
    			
				AnswerChoice answerChoice = question.getAnswerChoiceByValue(value);
				
				if (answerChoice != null) {
				  answer.setAnswerChoice(answerChoice);
				  QuestionGroup qg = answerChoice.getSubQuestions();
				  if (qg != null && qg.size() > 0) {
					answer.setSubAnswers(getResponsesForSubquestionPageGroup(parentGroup, qg.getGroupId(), sessionHistorySnapshot));  
				  }
				}
				  //TODO: currently does not support subquestions: see SurveyAnswerRetrieverRequestImpl.
				questionAnswers.add(answer);
			  }
    		  groupAnswers.add(questionAnswers);
    		}
		}
      return groupAnswers;	
    }
    
    /**
     * {@inheritDoc}
     */
    public GroupAnswers getResponsesForPageGroup(String pageGroup, Long surveyId) {
        
        Connection conn = null;
        try {
          conn = getConnection(false);
          return getResponsesForPageGroup(pageGroup, surveyId, conn);
        } catch (SQLException sqle) {
            logger.error("Error occurred getting responses for page group " + 
                    pageGroup  + " and mapping to " +
                    "group answers.", sqle);
            throw new DataRetrievalFailureException(sqle.getMessage(), sqle);
        } finally {
            DatabaseUtilities.cleanup(conn);
        }
        
    }
    
    /**
     * {@inheritDoc}
     */
    public GroupAnswers getResponsesForPageGroup(String pageGroup, Long surveyId, Connection conn) {
        
        PreparedStatement statement = null;
        ResultSet rs = null;
        
        GroupAnswers answerSet = new GroupAnswers(pageGroup);                              
        
        try {
            statement = conn.prepareStatement(RESPONSES_BY_PAGE_GROUP, 
            		ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            
            if (logger.isDebugEnabled())
              logger.debug("SQL QUERY = [ " + statement.toString() + "]");
            statement.setLong(1, surveyId);
            statement.setString(2, pageGroup);
            
            
            rs = statement.executeQuery();
            
            while (rs.next()) {
                long questionId = rs.getLong("QUESTION_ID");
                String questionKey = rs.getString("QUESTION_KEY");
                
                QuestionAnswers qa = answerSet.getAnswers(questionId);
                if (qa == null) {
                   qa = new QuestionAnswers(new Question(questionId, questionKey));
                   answerSet.add(qa);
                }

                qa.add(readAnswer(qa.getQuestion(), rs, conn));
            }
            
            conn.commit();
            
        } catch (SQLException sqle) {
            logger.error("Error occurred getting responses for page group " + 
                    pageGroup  + " and mapping to " +
                    "group answers.", sqle);
            throw new DataRetrievalFailureException(sqle.getMessage(), sqle);
        }finally{
            DatabaseUtilities.cleanup(rs, statement);
        }
        
        return answerSet;
    }
    
    /**
     * Reads an answer from the current row of the given result set, and retrieves the subanswers
     * for that answer.
     * 
     * @param question the question being answered.
     * @param rs the result set containing the data.
     * @param conn the database connection used to retrieve the answer.
     * 
     * @return an answer read from the current row of the result set.
     * 
     * @throws SQLException thrown if there is an error while reading the result set.
     */
    private SurveyAnswer readAnswer(Question question, ResultSet rs, Connection conn) throws SQLException {
       
       String answerKey = rs.getString("ANSWER_KEY");
       Long answerChoiceId = rs.getLong("ANSWER_CHOICE_ID");
       Long subquestionGroupId = rs.getLong("SUBQUESTION_GROUP_ID");
       
       SurveyAnswer surveyAnswer = new SurveyAnswer(answerKey);
       
       if (answerChoiceId != null) {
       	AnswerChoice answerChoice = new AnswerChoice(answerChoiceId, answerKey);
       	surveyAnswer.setAnswerChoice(answerChoice);
       }
       
       if (subquestionGroupId != null) {
       	if (subquestionGroupId > 0) {
       	  if (surveyAnswer.getSubAnswers() == null)
       		surveyAnswer.setSubAnswers(new GroupAnswers(subquestionGroupId));
       	
       	  while (rs.next()) {
       		 long currentQuestionGroupId = rs.getLong("QUESTION_GROUP_ID");
       		 if (logger.isDebugEnabled()) {
       			 logger.debug("Checking for subquestion previous responses: CURRENT QUESTION GROUP ID = [ " +
       				currentQuestionGroupId + " ] and sub question group being iterated = [ " + subquestionGroupId + " ]");
       		 }
       		 if (currentQuestionGroupId == subquestionGroupId.longValue()) {
       		   readSubAnswer(rs, surveyAnswer.getSubAnswers(), subquestionGroupId);
       		 } else {
       			 rs.previous();
       			 break;
       		 }
       	  }
       	}
       }
       
       return surveyAnswer;
    }
    
    protected void readSubAnswer(ResultSet rs, GroupAnswers subgroupAnswers, Long currentSubquestionGroupId) throws SQLException {
    	
    	Long questionGroupId = rs.getLong("QUESTION_GROUP_ID");
    	if (questionGroupId.equals(currentSubquestionGroupId)) {
    		
            String questionKey = rs.getString("QUESTION_KEY");
            Long questionId = rs.getLong("QUESTION_ID");
            Long answerChoiceId = rs.getLong("ANSWER_CHOICE_ID");
            String answerKey = rs.getString("ANSWER_KEY");
            
            Question question = new Question(questionId, questionKey);
            
            SurveyAnswer surveyAnswer = new SurveyAnswer(answerKey);
            
            if (answerChoiceId != null) {
            	AnswerChoice answerChoice = new AnswerChoice(answerChoiceId, answerKey);
            	surveyAnswer.setAnswerChoice(answerChoice);
            }
      
            QuestionAnswers questionAnswers = new QuestionAnswers(question);
            questionAnswers.add(surveyAnswer);
            subgroupAnswers.add(questionAnswers);
    	} 
    }
    
    protected void readResponseSerialResult(ResultSet rs, 
            GroupBasedAnswerSet answerSet, boolean localizeAnswerGroups, ResourceBundle bundle) throws SQLException {
     
    	Map<GroupAnswersKey, List<QuestionAnswerSubquestionPairDto>> recordHolder = new TreeMap<GroupAnswersKey, List<QuestionAnswerSubquestionPairDto>>();
    	
    	while (rs.next()) {
    	  QuestionAnswerSubquestionPairDto questionAnswerRecord = new QuestionAnswerSubquestionPairDto();
    	  questionAnswerRecord.setAnswerResponseSetId(rs.getLong("RESPONSE_SET_ID"));
    	  questionAnswerRecord.setQuestionGroupId(rs.getLong("QUESTION_GROUP_ID"));
    	  questionAnswerRecord.setQuestionGroup(rs.getString("QUESTION_GROUP"));
    	  int questionSequence = rs.getInt("QUESTION_SEQUENCE");
    	  if (!rs.wasNull()) {
    		  questionAnswerRecord.setQuestionSequence(new Integer(questionSequence));
    	  }
    	  questionAnswerRecord.setQuestionKey(rs.getString("QUESTION_KEY"));
    	  questionAnswerRecord.setQuestionId(rs.getLong("QUESTION_ID"));
    	  questionAnswerRecord.setAnswerChoiceId(rs.getLong("ANSWER_CHOICE_ID"));
          int answerChoiceSequence = rs.getInt("CHOICE_SEQUENCE");
          if (!rs.wasNull()) {
        	  questionAnswerRecord.setAnswerChoiceSequence(new Integer(answerChoiceSequence));
          }
          questionAnswerRecord.setAnswerId(rs.getLong("SURVEY_ANSWER_ID"));
          questionAnswerRecord.setAnswerKey(rs.getString("ANSWER_KEY"));
          questionAnswerRecord.setSubquestionGroupId(rs.getLong("SUBQUESTION_GROUP_ID"));
          questionAnswerRecord.setSubquestionResponseSetId(rs.getLong("SUBQUESTION_RESPONSE_SET_ID"));
    	
          //Construct a GroupAnswersKey to see if this group already contained in structure.
          GroupAnswersKey groupAnswersKey = new GroupAnswersKey(questionAnswerRecord.getQuestionGroupId(), questionAnswerRecord.getAnswerResponseSetId());
          
          if (recordHolder.containsKey(groupAnswersKey)) {
        	  List<QuestionAnswerSubquestionPairDto> questionAnswers = recordHolder.get(groupAnswersKey);
              questionAnswers.add(questionAnswerRecord);
          } else {
        	 List<QuestionAnswerSubquestionPairDto> questionAnswers = new ArrayList<QuestionAnswerSubquestionPairDto>();
        	 questionAnswers.add(questionAnswerRecord);
        	 recordHolder.put(groupAnswersKey, questionAnswers);
          }
    	} //end while: rs processing
    	
    	transformDtosToGroupBasedAnswerSet(recordHolder, answerSet, localizeAnswerGroups, bundle);
			
    }
    
    /**
     * 
     * @param recordHolder
     * @param answerSet
     * @param localizeAnswerGroups
     * @param bundle
     */
    protected void transformDtosToGroupBasedAnswerSet(Map<GroupAnswersKey, List<QuestionAnswerSubquestionPairDto>> recordHolder, 
    		GroupBasedAnswerSet answerSet,
    		boolean localizeAnswerGroups, 
    		ResourceBundle bundle) {
    	
    	Set<Map.Entry<GroupAnswersKey, List<QuestionAnswerSubquestionPairDto>>> allRecordEntries = recordHolder.entrySet();
    	Set<GroupAnswersKey> processedSubquestionGroupAnswers = new HashSet<GroupAnswersKey>();
    	
    	for (Map.Entry<GroupAnswersKey, List<QuestionAnswerSubquestionPairDto>> entry : allRecordEntries) {
			GroupAnswersKey groupAnswersKey = entry.getKey();
			
		    if(processedSubquestionGroupAnswers.contains(groupAnswersKey)) {
		    	//indicates processed subquestion group, so skip
		    } else {
		    	List<QuestionAnswerSubquestionPairDto> currentGroupAnswersDtoList = entry.getValue();
		    	//Process all the items in the dto list for the group
		    	GroupAnswers groupAnswers = processDtoListForGroupAnswers(groupAnswersKey, currentGroupAnswersDtoList, 
		    			answerSet, 
		    			recordHolder, processedSubquestionGroupAnswers, 
		    			localizeAnswerGroups, bundle);
		    	answerSet.addToGroupBasedAnswerSet(groupAnswersKey, groupAnswers);
		    }
		}
    }
    
    protected GroupAnswers processDtoListForGroupAnswers(final GroupAnswersKey groupAnswersKey, 
    		List<QuestionAnswerSubquestionPairDto> currentGroupAnswersDtoList,
    		GroupBasedAnswerSet answerSet,
    		Map<GroupAnswersKey, List<QuestionAnswerSubquestionPairDto>> recordHolder,
    		Set<GroupAnswersKey> processedSubquestionGroupAnswers,
    		boolean localizeAnswerGroups, 
    		ResourceBundle bundle) {
    	
    	GroupAnswers currentGroupAnswers = null;
    	Question question = null;
    	QuestionAnswers questionAnswers = null;
    	Long currentQuestionId = null;
    	
    	//In some cases the group answers dto list will be null
    	//subgroup of checkboxes where no checkboxes are selected, therefore just return null (null GroupAnswers)
    	if (currentGroupAnswersDtoList == null) return null;
    	
    	for (QuestionAnswerSubquestionPairDto pairDto : currentGroupAnswersDtoList) {
    	  String questionKey = pairDto.getQuestionKey();
     		 
    	  //Create new question if 1st question in set or new question in set.
    	  if(question == null || !currentQuestionId.equals(pairDto.getQuestionId())) {
    	    currentQuestionId = pairDto.getQuestionId();
    		question = new Question(currentQuestionId, questionKey, getLocalizedQuestionText(questionKey, bundle));
    		questionAnswers = new QuestionAnswers(question);
    	  }
    	  
   	     SurveyAnswer surveyAnswer = localizeSurveyAnswer(pairDto.getAnswerId(), pairDto.getAnswerKey(), bundle);
		
   	        if (answerSet.containsGroupAnswers(groupAnswersKey)) {
   	          currentGroupAnswers = answerSet.getGroupAnswers(groupAnswersKey);	
   	        } else {
   	          currentGroupAnswers = new GroupAnswers(pairDto.getQuestionGroupId());
   	          if (localizeAnswerGroups)
   	            currentGroupAnswers.setLocalizedGroupLabel(getLocalizedGroup(pairDto.getQuestionGroup(), bundle));
   	        }
   	        
   	        //Set answer choice id on survey answer if present.
   	        Long answerChoiceId = pairDto.getAnswerChoiceId();
   	        if (answerChoiceId != null && (!answerChoiceId.equals(new Long(0)))) {
   	        	AnswerChoice answerChoice = new AnswerChoice(answerChoiceId, pairDto.getAnswerKey());
   	        	surveyAnswer.setAnswerChoice(answerChoice);
   	        }
   	        
   	        Long subquestionGroupId = pairDto.getSubquestionGroupId();
   	        if (subquestionGroupId != null) {
   	        	if (subquestionGroupId > 0) {
   	        	
   	        	  //Create GroupAnswerKey based on subquestion metadata (group id and response set id.
   	        	  GroupAnswersKey subquestionGroupAnswersKey = new GroupAnswersKey(pairDto.getSubquestionGroupId(), pairDto.getSubquestionResponseSetId());
   	        	  
   	        	  //Retrieve list of dtos for subanswer group from the dto records holder map
   	        	  List<QuestionAnswerSubquestionPairDto> subquestionGroupAnswers = recordHolder.get(subquestionGroupAnswersKey);
   	        	  //Indicate that subquestion group has been processed
   	              processedSubquestionGroupAnswers.add(subquestionGroupAnswersKey);	  
   	        	  
   	              //Recursively called to process subquestion group answers.
   	              GroupAnswers subgroupAnswers = processDtoListForGroupAnswers(subquestionGroupAnswersKey, subquestionGroupAnswers, 
   	            		  answerSet, 
   	            		  recordHolder, processedSubquestionGroupAnswers, 
   	            		  localizeAnswerGroups, bundle);
   	              
   	                if (surveyAnswer.getSubAnswers() == null)
  	        		  surveyAnswer.setSubAnswers(subgroupAnswers);
   	        	}
   	        }
   	        
   	        if (surveyAnswer != null)
   	        	questionAnswers.add(surveyAnswer);
    	} //end for loop
    	
    	currentGroupAnswers.add(questionAnswers);
    	
    	return currentGroupAnswers;
    }
    
    /**
     * Localized the encoded question text for user friendly display.
     * @param questionKey the question key to localize.
     * @param bundle the resource bundle yo use for localization
     * @return the localized version of the question key for text display.
     */
    protected String getLocalizedQuestionText(String questionKey, ResourceBundle bundle) {
    	return ResourceUtilities.getLocalizedResourceEntry(questionKey, bundle);
    }
    
    /**
     * Localizes the encoded group name for user friendly display.
     * @param group the group name to localize.
     * @return the localized string representation of the passed name.
     */
    protected String getLocalizedGroup(String group, ResourceBundle bundle) {
        String groupDecodedValue = null;
        try {
            groupDecodedValue = bundle.getString(group);
         } catch (MissingResourceException mre) {
             if (logger.isDebugEnabled())
             logger.debug("Missing resource exception occurred will use" +
                     "straight value for group [ " + group + " ]");
         }
         if (groupDecodedValue ==  null) { 
             groupDecodedValue = group;
         }
        return groupDecodedValue;
    }
    
    /**
     * Localizes the survey answer by decoding the passed answer key then returns the 
     * initialized <object>SurveyAnswer</object> instance.
     * @param answerId the answer id for the answer.
     * @param answerKey the key for the answer.
     * @param bundle the bundle to be used to convert the key to a localized version.
     * @return the initiated <object>SurveyAnswer</object> instance.
     */
    protected SurveyAnswer localizeSurveyAnswer(Long answerId, String answerKey, ResourceBundle bundle) {
    	
    	String localizedAnswerValue = ResourceUtilities.getLocalizedResourceEntry(answerKey, bundle);
    	if (localizedAnswerValue == null) return null;
    	SurveyAnswer answerItem = new SurveyAnswer(answerId, answerKey, localizedAnswerValue);
    	return answerItem;
    }
        
    /** Injection setters */
    public void setQuestionDao(QuestionDao questionDao) {
        this.questionDao = questionDao;
    }
}
