package org.jharks.workflow.survey.engine.dao.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.jharks.workflow.Answer;
import org.jharks.workflow.AnswerContext;
import org.jharks.workflow.AnswerKey;
import org.jharks.workflow.AnswerProperty;
import org.jharks.workflow.AnswerValueHandler;
import org.jharks.workflow.AnswerValueHandlerFactory;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.exception.MissingValueHandlerException;
import org.jharks.workflow.survey.engine.dao.QuestionDao;
import org.jharks.workflow.survey.engine.dao.SurveyResponsePersistenceDao;
import org.jharks.workflow.survey.engine.gui.taglib.PresentationConstants;
import org.jharks.workflow.survey.engine.model.Survey;
import org.jharks.workflow.survey.engine.model.SurveyStatus;
import org.jharks.workflow.survey.util.DatabaseUtilities;
import org.jharks.workflow.survey.util.SqlUtilities;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;

/**
 * Handles the creation of a survey DB record and persistence of responses
 * gathered during the survey.
 * 
 * @author Michael J. Rizzo
 */
public class SurveyResponseJdbcPersistenceDaoImpl extends AbstractJdbcDaoImpl
		implements SurveyResponsePersistenceDao {

	/** The logger instance for this class. */
	private static final Logger LOGGER = Logger
			.getLogger(SurveyResponseJdbcPersistenceDaoImpl.class);

	/** The SQL query used to insert survey details. */
	private static final String CREATE_SURVEY_SQL = "INSERT INTO "
			+ DATABASE_SCHEMA 
		    + "SURVEY (SURVEY_ID, SURVEY_DATE_TIME, "
			+ "CREATED_BY_USER_ID, STATUS, RESOURCE_BUNDLE) VALUES (?, ?, ?, ?, ?)";

	/** Query used to look up a user id. */
	//private static final String GET_USER_ID_SQL = "SELECT USER_ID FROM " + DATABASE_SCHEMA + "APP_USER "
	//		+ "WHERE USER_NAME = UPPER(?)";

	/** Query used to update a survey record timestamp. */
	private static final String UPDATE_SURVEY_RECORD = "UPDATE " + DATABASE_SCHEMA + "SURVEY SET "
			+ "UPDATED_DATE_TIME = ?, UPDATED_BY_USER_ID = ?, STATUS = ? WHERE SURVEY_ID = ?";

	private static final String RETRIEVE_QUESTIONS_FOR_GROUP = "(SELECT "
			+ "Q.QUESTION_KEY FROM " 
			+ DATABASE_SCHEMA + "QUESTION Q," 
			+ DATABASE_SCHEMA + "QUESTION_GROUP QG, "
			+ DATABASE_SCHEMA + "QUESTION_GROUP_QUESTION QGQ"
			+ "WHERE "
			+ "QGQ.QUESTION_ID = Q.QUESTION_ID AND QGQ.QUESTION_GROUP_ID = QG.QUESTION_GROUP_ID " 
			+ "AND QG.QUESTION_GROUP = ?)";

	private static final String INSERT_RESPONSE_SET_SQL = "INSERT INTO "
		    + DATABASE_SCHEMA + "SURVEY_RESPONSE_SET (RESPONSE_SET_ID, SURVEY_ID, "
			+ "QUESTION_ID, QUESTION_KEY, SUBMISSION_DATETIME) "
			+ "VALUES (?, ?, ?, ?, ?)";

	/** The SQL query used to insert answer values. */
	private static final String INSERT_ANSWER_SQL = "INSERT INTO "
		    + DATABASE_SCHEMA + "SURVEY_ANSWER (SURVEY_ANSWER_ID, RESPONSE_SET_ID, "
			+ "SURVEY_ID, ANSWER_DATA_TYPE_ID, ANSWER_KEY) "
			+ "VALUES (?, ?, ?, ?, ?)";

	/** The SQL query used to insert answer properties. */
	private static final String INSERT_ANS_PROP_SQL = "INSERT INTO "
		    + DATABASE_SCHEMA + "ANSWER_PROPERTY (ANSWER_PROPERTY_ID, "
			+ "RESPONSE_VALUE_ID, SURVEY_ID, PROPERTY_NAME, PROPERTY_VALUE) "
			+ "VALUES (?, ?, ?, ?, ?)";
	
	/** The SQL query used to delete all response sets from a survey. */
	private static final String DELETE_SURVEY_RESPONSE_SETS = 
		    "DELETE FROM " + DATABASE_SCHEMA + "SURVEY_RESPONSE_SET WHERE SURVEY_ID = ?";
	
	/** The SQL used to change status of 1 to n survey records. */
	private static final String UPDATE_SURVEY_STATUS = "UPDATE " 
		    + DATABASE_SCHEMA + "SURVEY SET STATUS = :statusCode WHERE SURVEY_ID IN ( :surveyIds )";
	
	/** The SQL string used to retrieve a survey record. */
	private static final String SELECT_SURVEY_RECORD = "SELECT SURVEY_ID, SURVEY_DATE_TIME, FINALIZED_DATE_TIME, "
		    + "STATUS, UPDATED_DATE_TIME, RESOURCE_BUNDLE FROM " + DATABASE_SCHEMA + "SURVEY WHERE SURVEY_ID = ?";

	/** The injected question dao instance. */
	private QuestionDao questionDao;
	
	/**
	 * {@inheritDoc}
	 * @deprecated
	 */
	public Long createSurvey(String userName) throws SQLException {
		return createSurvey(userName, null);
	}
    
	/**
	 * {@inheritDoc}
	 */
	public Long createSurvey(String userName, String resourceBundleName) throws SQLException {
		Connection conn = null;
		try {
			conn = getConnection(false);

			return createSurvey(userName, resourceBundleName, conn);
		} catch (SQLException sqle) {
			LOGGER.error("Error occurred creating survey id. ", sqle);
			throw sqle;
		} finally {
			DatabaseUtilities.cleanup(conn);
		}

	}
	
	public Long createSurvey(String userName, String resourceBundleName, Connection conn)
			throws SQLException {

		Long surveyKey = null;

		try {
			surveyKey = new Long(DatabaseUtilities.getNextValueInSequence(
					DATABASE_SCHEMA + "SURVEY_ID_SEQUENCE", conn));

			Long systemUserId = getSystemUserId(conn, userName);
			if (systemUserId == null)
				throw new SQLException(
						"No matching system user id "
								+ "was found for the logged in user. Record can not be created.");
			
			if (LOGGER.isDebugEnabled()) LOGGER.debug("CREATE_SURVEY_SQL String = [ " + CREATE_SURVEY_SQL  + " ]");
			
			PreparedStatement pstmt = conn.prepareStatement(CREATE_SURVEY_SQL);
			pstmt.setLong(1, surveyKey);
			pstmt.setTimestamp(2, new Timestamp((new Date()).getTime()));
			pstmt.setLong(3, systemUserId);
			pstmt.setInt(4, SurveyStatus.INCOMPLETE.getId());
			if (resourceBundleName == null || resourceBundleName.trim().equals("")) {
				pstmt.setNull(5, Types.VARCHAR);
			} else {
				pstmt.setString(5, resourceBundleName);
			}
			pstmt.executeUpdate();
			pstmt.close();

			conn.commit();
		} catch (SQLException sqle) {
			LOGGER.error("Error" + " occurred creating survey key.", sqle);
			conn.rollback();
			throw sqle;
		} finally {
			DatabaseUtilities.cleanup(conn);
		}

		return surveyKey;
	}
	
	/**
	 * {@inheritDoc}
	 */
    public Survey getSurveyRecord(Long surveyId) {
       
    	return (Survey) 
    	  this.getJdbcTemplate().queryForObject(SELECT_SURVEY_RECORD, new Object[] {surveyId}, new SurveyRowMapper());
   
    }
	
    private final class SurveyRowMapper implements ParameterizedRowMapper<Survey> {
    
    	public Survey mapRow(ResultSet rs, int rowNum) throws SQLException {
          Survey survey = new Survey();
          
          Long surveyId = rs.getLong("SURVEY_ID");
          survey.setSurveyId(surveyId);
          survey.setStartDate(rs.getTimestamp("SURVEY_DATE_TIME"));
          survey.setEndDate(rs.getTimestamp("FINALIZED_DATE_TIME"));
          survey.setStatus(SurveyStatus.valueOf(rs.getInt("STATUS")));
          survey.setResourceBundleName(rs.getString("RESOURCE_BUNDLE"));
          
          return survey;
    	}
    }
    
	/**
	 * {@inheritDoc}
	 */
	public Survey persistSurvey(Survey survey, SessionHistory history)
			throws SQLException {

		Connection conn = getConnection(false);
		try {
			updateRecordStatus(survey, history, conn);
			saveResponses(survey, history, conn);
			conn.commit();
		} catch (SQLException sqle) {
			LOGGER.error("Error" + " occurred persisting survey responses: "
					+ " will commit rollback", sqle);
			conn.rollback();
			throw sqle;
		} finally {
			DatabaseUtilities.cleanup(conn);
		}

		return survey;
	}

	private void updateRecordStatus(Survey survey, SessionHistory history,
			Connection conn) throws SQLException {

		PreparedStatement pstmt = null;

		try {
			String userName = (String) history.getProperty("user_name");

			Long systemUserId = getSystemUserId(conn, userName);
			if (systemUserId == null)
				throw new SQLException(
						"No matching system user id"
								+ "was found for the logged in user. Record can not be created.");

			pstmt = conn.prepareStatement(UPDATE_SURVEY_RECORD);
			pstmt.setTimestamp(1, new Timestamp((new Date()).getTime()));
			pstmt.setLong(2, systemUserId);
			pstmt.setInt(3, survey.getStatus().getId());
			pstmt.setLong(4, survey.getSurveyId());

			pstmt.executeUpdate();
		} catch (SQLException sqle) {
			LOGGER.error("Error occurred updating record timestamp.", sqle);
			conn.rollback();
			throw sqle;
		} finally {
			DatabaseUtilities.cleanup(pstmt);
		}

	}

	/**
	 * {@inheritDoc}
	 */
	public void saveResponses(Survey survey, SessionHistory history) {
		// TODO Auto-generated method stub, consider removing.
	}

	/**
	 * Iterates through each response set on the <CODE>SessionHistory</CODE>
	 * instance and persists to a database.
	 * 
	 * @param survey
	 *            the survey record containing a valid survey identifier.
	 * @param history
	 *            the <CODE>SessionHistory</CODE> instance of interest.
	 * @param conn
	 *            a database connection.
	 * @throws SQLException
	 *             if error occurs persisting any of the response sets.
	 */
	public void saveResponses(Survey survey, SessionHistory history,
			Connection conn) throws SQLException {
		Set<AnswerKey> keys = history.getResponseKeys();
		Iterator<AnswerKey> responses = keys.iterator();

		// TODO: Currently have no way to set deleteAllSurveyResponseSets to anything but true.
		boolean deleteAllSurveyResponseSets = true;
		String groupToDelete = (String) history
					.getProperty(PresentationConstants.QUESTION_GROUP_ID);
		if (groupToDelete != null) {
			deleteResponses(survey, groupToDelete, deleteAllSurveyResponseSets, conn);
		}

		while (responses.hasNext()) {
			AnswerKey key = (AnswerKey) responses.next();

			List<Answer> answerObjects = history.getResponses(key, null);

			saveResponse(key.getQuestionId(), survey, answerObjects, conn);
		}
	}

	/**
	 * Persists record into survey question table and then proceeds to persist
	 * response set.
	 * 
	 * @param key
	 *            the response set key.
	 * @param survey
	 *            a <CODE>Survey</CODE> instance.
	 * @param answers
	 *            a list of <CODE>SurveyAnswer</CODE> instances.
	 * @param conn
	 *            a database connection.
	 * @throws SQLException
	 *             if error occurs persisting the response set.
	 */
	private void saveResponse(String key, Survey survey, List<Answer> answers,
			Connection conn) throws SQLException {

		if (survey.getStartDate() == null)
			survey.setStartDate(new Date());

		if (answers != null && answers.size() > 0)
			saveResponseSet(key, survey, answers, conn);
	}

	/**
	 * Persists the response set.
	 * 
	 * @param key
	 *            the question key.
	 * @param survey
	 *            a <CODE>Survey</CODE> instance.
	 * @param answers
	 *            a list of <CODE>SurveyAnswer</CODE> instances.
	 * @param conn
	 *            a database connection.
	 * @throws SQLException
	 *             if error occurs persisting the reponse set.
	 */
	private void saveResponseSet(String key, Survey survey, List<Answer> answers,
			Connection conn) throws SQLException {

		PreparedStatement pstmt = conn
				.prepareStatement(INSERT_RESPONSE_SET_SQL);

		long responseSetId = DatabaseUtilities.getNextValueInSequence(DATABASE_SCHEMA + "SURVEY_RESP_SET_ID_SEQUENCE", conn);
		
		pstmt.setLong(1, responseSetId);
		pstmt.setLong(2, survey.getSurveyId());
		pstmt.setLong(3, questionDao.getLoadedQuestionId(key));
		pstmt.setString(4, key);
		pstmt.setTimestamp(5, new Timestamp(survey.getStartDate().getTime()));

		pstmt.executeUpdate();
		pstmt.close();

		saveAnswers(responseSetId, survey, answers, conn);

	}
	
	/**
	 * {@inheritDoc}
	 */
	public int changeSurveyStatus(List<Long> surveyIds, SurveyStatus status) {
		
		NamedParameterJdbcTemplate namedParameterJdbcTemplate = 
			new NamedParameterJdbcTemplate(this.getJdbcTemplate().getDataSource());
		
		MapSqlParameterSource parameters = new MapSqlParameterSource();
		parameters.addValue("statusCode", status.getId());
		parameters.addValue("surveyIds", surveyIds);

		
		return namedParameterJdbcTemplate.update(UPDATE_SURVEY_STATUS, parameters);
	}

	/**
	 * Persists the <CODE>SurveyAnswer</CODE> instances within the specified response
	 * set.
	 * 
	 * @param responseSetId
	 *            the auto generated key for the response set.
	 * @param survey
	 *            a <CODE>Survey</CODE> instance.
	 * @param answers
	 *            a list of <CODE>SurveyAnswer</CODE> instances.
	 * @throws SQLException
	 *             if error occurs persisting answer values.
	 */
	private void saveAnswers(long responseSetId, Survey survey,
			List<Answer> answers, Connection conn) throws SQLException {

		// Have seen issues when using batch updates and regular updates
		// on same connection. However MySQL seems to be handling them and
		// actually enters deadlock if you try to use two separate connections.
		try {

			PreparedStatement pstmt = conn.prepareStatement(INSERT_ANSWER_SQL);

			Iterator<Answer> answerIt = answers.iterator();
			ArrayList<Long> responseValueIds = new ArrayList<Long>();

			while (answerIt.hasNext()) {
				Answer answer = (Answer) answerIt.next();

				String valueString = getValueString(answer.getValue());
                Long answerId = DatabaseUtilities.getNextValueInSequence(DATABASE_SCHEMA + "SURVEY_ANSWER_ID_SEQUENCE", conn);
				
				pstmt.setLong(1, answerId);
				pstmt.setLong(2, responseSetId);
				pstmt.setLong(3, survey.getSurveyId());
				// TODO: currently abstract type not being used.
				pstmt.setInt(4, 1);
				if (valueString == null) {
					pstmt.setNull(5, Types.VARCHAR);
				} else {
					pstmt.setString(5, valueString);
				}
				pstmt.executeUpdate();
				
				if (LOGGER.isDebugEnabled())
					LOGGER.debug("Add response to batch: last id = " + "["
							+ answerId + "]");
				responseValueIds.add(answerId);
			}

			pstmt.close();

			// TODO: Currently there are no answer properties being used and no
			// answer properties table exists.

			// saveAnswerProperties(survey.getSurveyId(), responseValueIds,
			// answers, conn);

		} catch (SQLException sqle) {
			LOGGER.error("Error" + " occurred persisting answer objects as "
					+ " batch: will commit rollback", sqle);
			throw sqle;
		}
	}

	/**
	 * Deletes existing records from DB for a particular work flow. Used by
	 * <code>PersistenceManager</code> before proceeding to persitence of new
	 * answers.
	 * 
	 * @param survey
	 *            The existing survey object being worked on.
	 * @param grouping
	 *            The question group indicated.
	 * @param isDeleteAllSurveyResponseSets
	 *            indicates whether group is a flow grouping.
	 * @param conn
	 *            The pre fetched database connection.
	 * @throws SQLException
	 *             if error occurs deleting records.
	 */
	private void deleteResponses(Survey survey, String grouping,
			boolean isDeleteAllSurveyResponseSets, Connection conn) throws SQLException {
		PreparedStatement pstmt = null;
		Statement st = null;
		ResultSet rs = null;
		Collection<Object> ids = new ArrayList<Object>();
		int deleteUpdate = 0;

		try {
			st = conn.createStatement();
			
			if(isDeleteAllSurveyResponseSets) {
				//Delete all response sets for Survey
				pstmt = conn.prepareStatement(DELETE_SURVEY_RESPONSE_SETS);
				pstmt.setLong(1, survey.getSurveyId());
				deleteUpdate = pstmt.executeUpdate();
			} else {
			  //Delete only reponse sets for the current/specified question group.
			  StringBuffer sqlString = new StringBuffer("DELETE FROM "
					+ "SURVEY_RESPONSE_SET WHERE QUESTION_KEY IN ");

			  String innerSql = RETRIEVE_QUESTIONS_FOR_GROUP;
			  pstmt = conn.prepareStatement(innerSql);
			  pstmt.setString(1, grouping);
			  rs = pstmt.executeQuery();

			  while (rs.next()) {
				ids.add(rs.getString("QUESTION_KEY"));
			  }

			  if (ids.size() > 0) {
				sqlString.append(SqlUtilities.writeSqlVarcharSet(ids)).append(
						" AND SURVEY_ID = ").append(survey.getSurveyId());

				if (LOGGER.isDebugEnabled())
					LOGGER.debug("DELETE SQL STRING = [ "
							+ sqlString.toString() + " ]");

				deleteUpdate = st.executeUpdate(sqlString.toString());
				
				if (LOGGER.isDebugEnabled())
					LOGGER.debug("DELETED COUNT = [ "
							+ deleteUpdate + " ]");
			  }
			}

		} catch (SQLException sqle) {
			LOGGER.error("Error"
					+ " occurred trying to delete previous answers.", sqle);
		} finally {
			DatabaseUtilities.cleanup(st);
			DatabaseUtilities.cleanup(rs, pstmt);
		}
	}

	/**
	 * Persists the answer property values for a collection of
	 * <CODE>SurveyAnswer</CODE> instances.
	 * 
	 * @param answerIds
	 *            list of auto generated answer keys.
	 * @param answers
	 *            list of <CODE>SurveyAnswer</CODE> instances.
	 * @param conn
	 *            a database connection.
	 * @throws SQLException
	 *             if error occurs persisting answer properties.
	 */
	private void saveAnswerProperties(int studyNumber, List<Long> answerIds,
			List<Answer> answers, Connection conn) throws SQLException {

		PreparedStatement pstmt = conn.prepareStatement(INSERT_ANS_PROP_SQL);

		Iterator<Answer> answerIt = answers.iterator();
		int index = 0;
		boolean persistContext = false;

		while (answerIt.hasNext()) {
			Answer answer = (Answer) answerIt.next();
			AnswerContext anc = (AnswerContext) answer.getContext();

			if (anc != null && anc.getPropertySize() > 0) {
				batchContextRecord(pstmt, studyNumber, anc, (Long) answerIds
						.get(index));
				if (LOGGER.isDebugEnabled())
					LOGGER.debug("SurveyAnswer context to persist: passed id = "
							+ "[" + (Long) answerIds.get(index) + "]");
				persistContext = true;
			}

			index++;
		}
		if (persistContext)
			pstmt.executeBatch();

		pstmt.close();
	}

	/**
	 * Adds insert statement to answer property batch.
	 * 
	 * @param pstmt
	 *            a PrepareStatement instance.
	 * @param context
	 *            the passed <CODE>AnswerContext</CODE> instance.
	 * @param index
	 *            auto generated key for the corresponding response value.
	 * @throws SQLException
	 *             if error occurs setting a parameter or batching insert.
	 */
	private void batchContextRecord(PreparedStatement pstmt, int studyNumber,
			AnswerContext context, Long index) throws SQLException {

		Iterator<AnswerProperty> it = context.getPropertyIterator();

		while (it.hasNext()) {
			AnswerProperty ap = (AnswerProperty) it.next();
			if (LOGGER.isDebugEnabled())
				LOGGER.debug("Adding answer property for response value id: = "
						+ "[" + index.intValue() + "]");
			pstmt.setInt(1, index.intValue());
			pstmt.setInt(2, studyNumber);
			pstmt.setString(3, ap.getName());
			pstmt.setString(4, ap.getStringValue());
			pstmt.addBatch();
		}
	}

	/**
	 * This method will proces the object value and determine the most
	 * appropriate persistable format. If the <code>Object</code> is a
	 * <code>String</code> the method will simply return. If the
	 * <code>Object</code> is a member of a class that has a <code>
     * AnswerValueHandler</code>
	 * configured then the value will be processed by the handler. Lastly, the
	 * method will call the <code>toString</code> on the <code>Object</code> if
	 * it's not a <code>String</code> and no handler is found.
	 * 
	 * @param value
	 *            the value object to process.
	 * @return the most appopriate persistable format.
	 * @see org.jharks.workflow.AnswerValueHandler.
	 */
	private String getValueString(Object value) {
		if (value == null)
			return null;

		if (value instanceof String)
			return (String) value;

		try {
			AnswerValueHandler avh = AnswerValueHandlerFactory
					.getValueHandler(value);
			return avh.format(value);
		} catch (MissingValueHandlerException mvh) {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug(mvh.getMessage(), mvh);
			}
		} catch (Exception ex) {
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug(ex.getMessage(), ex);
			}
		}

		return value.toString();
	}

	/**
	 * Returns the system user id for specified username.
	 * 
	 * @param conn
	 *            The pre-fetched database connection.
	 * @param username
	 *            The username to retrieve.
	 * @return the matched user id or null if no matching entry.
	 */
	private Long getSystemUserId(Connection conn, String username) {
		//PreparedStatement pstmt = null;
		//ResultSet rs = null;
		Long systemUserId = null;
		//try {
		//	pstmt = conn.prepareStatement(GET_USER_ID_SQL);
		//	pstmt.setString(1, username);
		//	rs = pstmt.executeQuery();

		//	if (rs.next()) {
		//		systemUserId = rs.getLong("USER_ID");
		//	}
		//} catch (SQLException sqle) {
		//	LOGGER.error("Error"
		//			+ " occurred trying to retrieve system user id.", sqle);
		//} finally {
		//	DatabaseUtilities.cleanup(rs, pstmt);
		//}
		//TODO: Children's will pass a employee id. Clean up the method. Possible introduce an system user id extractor
		//configured through Spring that knows how to retrieve the user id.
		systemUserId = new Long(username);
		return systemUserId;
	}

	/**
	 * Retrieve the question dao instance.
	 * @return the question dao instance.
	 */
	public QuestionDao getQuestionDao() {
		return questionDao;
	}

	/**
	 * Set the question dao instance.
	 * @param question dao instance.
	 */
	public void setQuestionDao(QuestionDao questionDao) {
		this.questionDao = questionDao;
	}

	
	
}
