package org.jharks.workflow.survey.engine.dao.impl;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.jharks.workflow.survey.engine.dao.XmlWorkflowDefinitionDao;
import org.jharks.workflow.survey.engine.exception.SurveyEngineException;
import org.jharks.workflow.survey.util.DatabaseUtilities;
import org.springframework.jdbc.datasource.DataSourceUtils;

public class XmlWorkflowDefinitionDaoImpl extends AbstractJdbcDaoImpl implements
		XmlWorkflowDefinitionDao {

	private static Logger logger = Logger.getLogger(XmlWorkflowDefinitionDaoImpl.class);
	
	private static final String XML_DEFINITION_INSERT_QUERY = "INSERT INTO " + DATABASE_SCHEMA + "XML_WORKFLOW_DEFINITION (DEFINITION_ID, DEFINITION_NAME, DEFINITION_VERSION, DEFINITION_XML) VALUES (?, ?, ?, EMPTY_CLOB())";
	private static final String XML_DEFINITION_SELECT_FOR_UPDATE_QUERY = "SELECT DEFINITION_XML FROM " + DATABASE_SCHEMA + "XML_WORKFLOW_DEFINITION WHERE DEFINITION_ID = ? FOR UPDATE";
	private static final String XML_DEFINITION_SELECT_QUERY = "SELECT DEFINITION_XML FROM " + DATABASE_SCHEMA + "XML_WORKFLOW_DEFINITION WHERE DEFINITION_ID = ?";
	private static final String XML_DEFINITION_SELECT_LATEST_VERSION_QUERY = "SELECT DEFINITION_XML FROM " + DATABASE_SCHEMA + "XML_WORKFLOW_DEFINITION WHERE DEFINITION_NAME = ? AND DEFINITION_VERSION = " +
	        "(SELECT MAX(DEFINITION_VERSION) FROM " + DATABASE_SCHEMA + "XML_WORKFLOW_DEFINITION WHERE DEFINITION_NAME =?)";
	private static final String XML_DEFINITION_ID_LATEST_VERSION_QUERY = "SELECT DEFINITION_ID FROM " + DATABASE_SCHEMA + "XML_WORKFLOW_DEFINITION WHERE DEFINITION_NAME = ? AND DEFINITION_VERSION = " +
			"(SELECT MAX(DEFINITION_VERSION) FROM " + DATABASE_SCHEMA + "XML_WORKFLOW_DEFINITION WHERE DEFINITION_NAME =?)";
	private static final String XML_GET_DEFINITION_ID_QUERY = "SELECT DEFINITION_ID FROM " + DATABASE_SCHEMA + "XML_WORKFLOW_DEFINITION WHERE DEFINITION_NAME = ? AND DEFINITION_VERSION = ?";
	
	/**
	 * {@inheritDoc}
	 */
	public InputStream getXmlDefinitionAsStream(Long definitionId) {
		Connection conn = DataSourceUtils.getConnection(this.getDataSource());
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        InputStream xmlDefinitionRetrieved = null;
		
		try {
			  conn.setAutoCommit(false);

			  pstmt = conn.prepareStatement(XML_DEFINITION_SELECT_QUERY);
			  pstmt.setLong(1, definitionId);
			  
			  rs = pstmt.executeQuery();
			  
			  if (rs.next()) {
			  	xmlDefinitionRetrieved = DatabaseUtilities.getClobAsStream(rs, 1); 
			  }
		} catch (SQLException sqle) {
			throw new SurveyEngineException("Error occurred retrieving xml definition.", sqle);
		} finally {
			DatabaseUtilities.cleanup(conn, rs, pstmt);	  
	    }
		return xmlDefinitionRetrieved;
	}
	
	/**
	 * {@inheritDoc}
	 */
	public InputStream getXmlDefinitionLatestVersionAsStream(String definitionName) {
		Connection conn = DataSourceUtils.getConnection(this.getDataSource());
        PreparedStatement pstmt = null;
        ResultSet rs = null;
        InputStream xmlDefinitionRetrieved = null;
		
		try {
			  conn.setAutoCommit(false);

			  pstmt = conn.prepareStatement(XML_DEFINITION_SELECT_LATEST_VERSION_QUERY);
			  pstmt.setString(1, definitionName);
			  pstmt.setString(2, definitionName);
			  
			  rs = pstmt.executeQuery();
			  
			  if (rs.next()) {
			  	xmlDefinitionRetrieved = DatabaseUtilities.getClobAsStream(rs, 1); 
			  }
		} catch (SQLException sqle) {
			throw new SurveyEngineException("Error occurred retrieving xml definition.", sqle);
		} finally {
			DatabaseUtilities.cleanup(conn, rs, pstmt);	  
	    }
		return xmlDefinitionRetrieved;		
	}

	/**
	 * {@inheritDoc}
	 */
	public String getXmlDefinitionAsString(Long definitionId) {
		try {
		 return convertStreamToString(getXmlDefinitionAsStream(definitionId));
		} catch (IOException ioe) {
			throw new SurveyEngineException("Error occurred converting xml definition to string.", ioe);
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	public String getXmlDefinitionLatestVersionAsString(String definitionName) {
		try {
			 return convertStreamToString(getXmlDefinitionLatestVersionAsStream(definitionName));
			} catch (IOException ioe) {
				throw new SurveyEngineException("Error occurred converting xml definition to string.", ioe);
			}
	}

	/**
	 * {@inheritDoc}
	 */
	public void persistXmlDefinition(String definitionName, Long version, InputStream definitionContentStream) {
		Connection conn = DataSourceUtils.getConnection(this.getDataSource());
		  PreparedStatement pstmt = null;
		  ResultSet rs = null;
		  Clob clob = null;
		  Writer out = null;
			 try { 
				 conn.setAutoCommit(false);
			  pstmt = conn.prepareStatement(XML_DEFINITION_INSERT_QUERY);
			  Long definitionId = DatabaseUtilities.getNextValueInSequence(DATABASE_SCHEMA + "XML_WORKFLOW_DEF_ID_SEQUENCE", conn);
			  pstmt.setLong(1, definitionId);
			  pstmt.setString(2, definitionName);
			  pstmt.setLong(3, version);
			  pstmt.executeUpdate();
			  conn.commit();
			  
			  pstmt.close();
			  
			  pstmt = conn.prepareStatement(XML_DEFINITION_SELECT_FOR_UPDATE_QUERY);
			  pstmt.setLong(1, definitionId);
			  rs = pstmt.executeQuery();
			  
			  if(rs.next()) {
			  	clob = rs.getClob(1);
			  	
			    out = new BufferedWriter(new OutputStreamWriter(clob.setAsciiStream(0)));
			    IOUtils.copy(definitionContentStream, out);
			    
              out.flush();

			  conn.commit();
			  
			  }
			  
			} catch (IOException ioe) {
				logger.error("Error occured processing xml definition file stream", ioe);
				throw new SurveyEngineException("Error occured processing xml definition file stream", ioe);
			}
			 catch (SQLException sqle) {
				 throw new SurveyEngineException("Error occured persisting xml definition file stream", sqle);
			} finally {
				DatabaseUtilities.cleanup(conn, rs, pstmt);
				try {
				 out.close();
				} catch (Exception e) {}
			}
	}
	
	/**
	 * {@inheritDoc}
	 */
	public Long getDefinitionId(String definitionName, Long definitionVersion) {
		return this.getJdbcTemplate().queryForLong(XML_GET_DEFINITION_ID_QUERY, new Object[]{definitionName, definitionVersion});
	}
	
	/**
	 * {@inheritDoc}
	 */
    public Long getDefinitionIdLatestVersion(String definitionName) {
    	return this.getJdbcTemplate().queryForLong(XML_DEFINITION_ID_LATEST_VERSION_QUERY , new Object[]{definitionName, definitionName});
    }
    
    /**
     * Helper method that converts an <code>InputStream</code> to a String.
     * @param is the input stream to convert.
     * @return the converted string
     * @throws IOException if error occurs processing the input stream.
     */
    public String convertStreamToString(InputStream is) throws IOException {
		/*
		 * To convert the InputStream to String we use the Reader.read(char[]
		 * buffer) method. We iterate until the Reader return -1 which means
		 * there's no more data to read. We use the StringWriter class to
		 * produce the string.
		 */
		if (is != null) {
			Writer writer = new StringWriter();

			char[] buffer = new char[1024];
			try {
				Reader reader = new BufferedReader(new InputStreamReader(is,
						"UTF-8"));
				int n;
				while ((n = reader.read(buffer)) != -1) {
					writer.write(buffer, 0, n);
				}
			} finally {
				is.close();
			}
			return writer.toString();
		} else {
			return "";
		}
	}
}
