package org.jharks.workflow.survey.engine.gui.taglib;

import java.io.IOException;
import java.util.ResourceBundle;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.tagext.TryCatchFinally;

import org.apache.log4j.Logger;
import org.jharks.workflow.survey.engine.exception.SurveyEngineException;
import org.jharks.workflow.survey.engine.model.GroupBasedAnswerSet;
import org.jharks.workflow.survey.engine.model.Survey;
import org.jharks.workflow.survey.engine.service.ResponseManager;
import org.jharks.workflow.survey.engine.service.SurveyPersistenceManager;
import org.jharks.workflow.survey.engine.service.WorkFlowResourceBundleService;
import org.jharks.workflow.survey.engine.web.SurveyHtmlOutputBuilder;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

/**
 * 
 * A custom tag that knows how to iterate over a <code>GroupBasedAnswerSet</code>
 * and provided a formatted report output.
 * 
 * @author Michael J. Rizzo
 *
 */
public class SurveySummaryTag extends TagSupport implements TryCatchFinally {

	/** Serial version UID */
    private static final long serialVersionUID = 1L;
    
    /** The logger instance for this class. */
    private static Logger logger = Logger.getLogger(SurveySummaryTag.class);
	
	/** The surveyId to retrieve. */
	private String surveyId;
	
	/** The resource bundle base name. */
    private String resourceBundleBaseName;
	
	/**
	 * {@inheritDoc}
	 */
	@Override
    public int doEndTag() throws JspException {

		WebApplicationContext webAppContext = WebApplicationContextUtils.getWebApplicationContext(pageContext.getServletContext());
		ResponseManager responseManager = (ResponseManager) webAppContext.getBean("org.jharks.responseManager");
		
		WorkFlowResourceBundleService bundleService = (WorkFlowResourceBundleService) webAppContext.getBean("org.jharks.workFlowResourceBundleService");
	
		ResourceBundle bundle = null;
		if (getResourceBundleBaseName() == null) {
			SurveyPersistenceManager surveyManager = (SurveyPersistenceManager) webAppContext.getBean("org.jharks.surveyPersistenceManager");
			Survey survey = surveyManager.getSurveyRecord(new Long(surveyId));
			if (survey.getResourceBundleName() != null)
	    	  bundle = bundleService.getWorkFlowResourceBundle(survey.getResourceBundleName());
	        if (bundle == null) 
	          bundle = bundleService.getWorkFlowResourceBundle(PresentationConstants.DEFAULT_RESOURCE_BUNDLE_NAME);
		} else {
			bundle = bundleService.getWorkFlowResourceBundle(getResourceBundleBaseName());
		}

	    if (bundle == null) throw new SurveyEngineException("Resource bundle [ " + getResourceBundleBaseName() + " ] could not be found. Survey summary could not be generated");
	    if (logger.isDebugEnabled()) {
	      logger.debug("The survey id to summarize = [ " + surveyId + " ]");
	      logger.debug("The resource bundle base name to use to summarize = [ " + resourceBundleBaseName + " ]");
	    }
		
		GroupBasedAnswerSet answerSet = null;
		try {
			answerSet = responseManager.getResponsesSummary(surveyId, true, bundle);
		} catch (Throwable t) {
			logger.error("Error occurred retrieving summary from data store.", t);
	      	throw new JspException("Jharks SurveySummary Tag: Error occurred retrieving summary from data store: " + t.getMessage(), t);
		}
		
		//Get writer
	  	JspWriter writer = pageContext.getOut();
	    
	    SurveyHtmlOutputBuilder outputBuilder = new SurveyHtmlOutputBuilder();
	    
	    String output = outputBuilder.transformGroupBasedAnswerSet(answerSet, bundle);
		
	    try {
	      writer.print(output);
	    } catch (IOException ioe) {
	      	logger.error("Error occurred writing outut to jsp page.", ioe);
	      	throw new JspException("Jharks SurveySummaryTag: Error occurred writing output to jsp page: " + ioe.getMessage(), ioe);
	    }
	    
	    return EVAL_PAGE;
	}

	/**
	 * @return the surveyId
	 */
	public String getSurveyId() {
		return surveyId;
	}

	/**
	 * @param surveyId the surveyId to set
	 */
	public void setSurveyId(String surveyId) {
		this.surveyId = surveyId;
	}
	
	/**
	 * @return the resourceBundleBaseName
	 */
	public String getResourceBundleBaseName() {
		return resourceBundleBaseName;
	}

	/**
	 * @param resourceBundleBaseName the resourceBundleBaseName to set
	 */
	public void setResourceBundleBaseName(String resourceBundleBaseName) {
		this.resourceBundleBaseName = resourceBundleBaseName;
	}
	
	//the JSP container may cache and reuse tag handler objects.
	//this method releases instance variables so that the tag handler
	//can be reused afresh
	/**
	 * {@inheritDoc}
	 */
	public void release() {
	     surveyId = null;
	     resourceBundleBaseName = null;
	}
	
	public void doCatch(Throwable t){
	    try{
	        //print the exception message inside the JSP where the tag
	        //appears
	        pageContext.getOut().println(t.getMessage()+"<br />");
	      } catch (java.io.IOException ioException) {}
	}
	
	public void doFinally( ){
	   //do nothing here, since we don' t have any resources open
	   //like database connections
	}
}
