package org.jharks.workflow.survey.engine.gui.taglib;

import java.io.File;
import java.io.InputStream;
import java.io.StringReader;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.TagSupport;
import javax.servlet.jsp.tagext.TryCatchFinally;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.log4j.Logger;
import org.jharks.workflow.survey.engine.exception.SurveyEngineException;

/**
 * A simple tag used to perform the XSL transformation that renders the
 * survey questions.  To use this tag, be sure that the configuration file contains
 * the full path to the XSL file used to perform the transformation, in the
 * <CODE>survey-xsl-file</CODE> parameter.
 * <br><br>
 * This tag takes one of two parameters: <CODE>xml</CODE> or <CODE>xmlFile</CODE>.
 * If you are using the <CODE>xml</CODE> parameter, the parameter should be a string
 * containing the XML to be transformed.  If you are using the <CODE>xmlFile</CODE>
 * parameter, the parameter should be the full path to the XML file that is being
 * transformed.
 * 
 * @author Michael J. Rizzo
 * 
 */
public class XslTag extends TagSupport implements TryCatchFinally {

    private static final long serialVersionUID = 1L;
    
    private static Logger logger = Logger.getLogger(XslTag.class);
    
    /** The transformer used to perform the XSL transform. */
    private static Transformer transformer;     
    
    static {
        InputStream inStream = XslTag.class.getClassLoader().getResourceAsStream("/META-INF/survey.xsl");
    	TransformerFactory tf = TransformerFactory.newInstance();
        try {
           transformer = tf.newTransformer(new StreamSource(inStream));
        } catch (Exception e) {
              logger.error("Could not create a transformer.", e);
              throw new SurveyEngineException("Jharks Xsl Tag: Error occurred initializing the transformer factory. XSL file may be missing", e);
        }
    }
    
    /** The <CODE>xml</CODE> parameter. */
    private String xml;
    
    /** The <CODE>xmlFile</CODE> parameter. */
    private String xmlFile;
    
    /** The resource bundle base name. */
    private String resourceBundleBaseName;
    
    /** Whether to use db backed resources or not. */
    private String dbBackedResources = "false";
    
    /**
     * Performs the XSL transformation, writing the result of the transformation
     * to the output stream, and returning <CODE>SKIP_BODY</CODE>.
     * 
     */
    public int doEndTag() throws JspException {
       
       StreamSource source = null;
       if (xml != null) {
          source = new StreamSource(new StringReader(xml));
       } else if (xmlFile != null) {
          source = new StreamSource(new File(xmlFile));
       }
       
       if (transformer != null) {
          try {
        	 logger.debug("Checking XSL Tag attribute " + resourceBundleBaseName);
        	 logger.debug("Checking XSL Tag attribute " + dbBackedResources);
        	 transformer.setParameter("resource-bundle", resourceBundleBaseName);
        	 transformer.setParameter("db-backed-resource-bundles", dbBackedResources);
             transformer.transform(source, new StreamResult(pageContext.getOut()));
          } catch (Exception e) {
                logger.error("An error occurred while transforming XML.", e);
                throw new JspException("Jharks Xsl Tag: An error occurred while transforming XML for survey page: " + e.getMessage(), e);
          }
       }
       
       xml = null;
       xmlFile = null;
       
       return EVAL_PAGE;
    }
    
    /**
     * Sets the xml parameter.
     * 
     * @param xml the xml parameter.
     */
    public void setXml(String xml) {
       this.xml = xml;
    }
    
    /**
     * Returns the xml parameter.
     * 
     * @return the xml parameter.
     */
    public String getXml() {
       return xml;
    }
    
    /**
     * Sets the xmlFile parameter.
     * 
     * @param xmlFile the xmlFile parameter.
     */
    public void setXmlFile(String xmlFile) {
       this.xmlFile = xmlFile;
    }

    /**
     * Returns the xmlFile parameter.
     * 
     * @return the xmlFile parameter.
     */
    public String getXmlFile() {
       return xmlFile;
    }

	/**
	 * @return the resourceBundleBaseName
	 */
	public String getResourceBundleBaseName() {
		return resourceBundleBaseName;
	}

	/**
	 * @param resourceBundleBaseName the resourceBundleBaseName to set
	 */
	public void setResourceBundleBaseName(String resourceBundleBaseName) {
		this.resourceBundleBaseName = resourceBundleBaseName;
	}

	/**
	 * @return the dbBackedResources
	 */
	public String getDbBackedResources() {
		return dbBackedResources;
	}

	/**
	 * @param dbBackedResources the dbBackedResources to set
	 */
	public void setDbBackedResources(String dbBackedResources) {
		this.dbBackedResources = dbBackedResources;
	}
    
	//the JSP container may cache and reuse tag handler objects.
	//this method releases instance variables so that the tag handler
	//can be reused afresh
	/**
	 * {@inheritDoc}
	 */
	public void release() {
	     xml = null;
	     resourceBundleBaseName = null;
	     xmlFile = null;
	     dbBackedResources = "false";
	}
	
	public void doCatch(Throwable t){
	    try{
	        //print the exception message inside the JSP where the tag
	        //appears
	        pageContext.getOut().println(t.getMessage()+"<br />");
	      } catch (java.io.IOException ioException) {}
	}
	
	public void doFinally( ){
	   //do nothing here, since we don' t have any resources open
	   //like database connections
	}

    
}

