package org.jharks.workflow.survey.engine.model;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;


public class Question implements Serializable {

    private static final long serialVersionUID = 1L;

    /** The questionId generated by the database. */
    private Long questionId;
    
    /** The group ID. */
    private String groupId;
    
    /** The question key. */
    private String questionKey;
    
    /** The question text. */
    private String questionText;

    /** The name of the layout to use when displaying the question. */
    private String layoutName;
    
    /** A flag indicating whether the answer to this question should be saved. */
    private boolean saveAnswer;
    
    /** The answer choices. */
    private Collection<AnswerChoice> answerChoices = new LinkedList<AnswerChoice>();
    
    /** A <CODE>Map</CODE> containing the answer choices, keyed by value. */
    private Map<String, AnswerChoice> choicesByAnswerKey = new HashMap<String, AnswerChoice>();
    
    /**
     * Default public constructor.
     */
    public Question() {
    }
    
    /**
     * Constructor that initializes key and persistence based identifier.
     * @param questionId The question id to set.
     * @param questionKey The question key to set.
     */
    public Question(Long questionId, String questionKey) {
        this.questionId = questionId;
        this.questionKey = questionKey;
    }
    
    /**
     * Constructor that initializes key and text.
     * @param questionKey The question key to set.
     * @param questionText The question text to set.
     */
    public Question(String questionKey, String questionText) {
        this.questionKey = questionKey;
        this.questionText = questionText;
    }
    
    /**
     * Constructor that initializes key and persistence based identifier and text.
     * @param questionId The question id to set.
     * @param questionKey The question key to set.
     * @param questionText The question text to set.
     */
    public Question(Long questionId, String questionKey, String questionText) {
        this(questionId, questionKey);
        this.questionText = questionText;
    }
    
    
    /**
     * Return the question id.
     * 
     * @return the question id.
     */
    public Long getQuestionId() {
        return questionId;
    }

    /**
     * Set the question id.
     * 
     * @param questionId the question id to set.
     */
    public void setQuestionId(Long questionId) {
        this.questionId = questionId;
    }

    /**
     * Set the question group id.
     * 
     * @param groupId the question group id.
     */
    public void setGroupId(String groupId) {
        this.groupId = groupId;
    }

    /**
     * Return the question group id. 
     * 
     * @return the question group id.
     */
    public String getGroupId() {
        return groupId;
    }
    
    /**
     * Return the question key.
     * 
     * @return the question key.
     */
    public String getQuestionKey() {
        return questionKey;
    }

    /**
     * Set the question key.
     * 
     * @param questionKey the question key to set
     */
    public void setQuestionKey(String questionKey) {
        this.questionKey = questionKey;
    }

    /**
     * Return the question text.
     * 
     * @return the questionText
     */
    public String getQuestionText() {
        return questionText;
    }

    /**
     * Sets the question text.
     * 
     * @param questionText the questionText to set.
     */
    public void setQuestionText(String questionText) {
        this.questionText = questionText;
    }

    /**
     * Sets the name of the layout used to display this question.
     * 
     * @param layoutName the name of the layout used to display this question.
     */
    public void setLayoutName(String layoutName) {
       this.layoutName = layoutName;
    }
    
    /**
     * Returns the name of the layout used to display this question.
     * 
     * @return the name of the layout used to display this question.
     */
    public String getLayoutName() {
       return layoutName;
    }    
    
    /**
     * Sets a flag used to determine if the answer to this question should be saved.
     * 
     * @param saveAnswer a flag used to determine if the answer to this question should be saved.
     */
    public void setSaveAnswer(boolean saveAnswer) {
       this.saveAnswer = saveAnswer;
    }
    
    /**
     * Returns a flag used to determine if the answer to this question should be saved.
     * 
     * @return a flag used to determine if the answer to this question should be saved.
     */
    public boolean getSaveAnswer() {
       return saveAnswer;
    }
    
    /**
     * Adds an answer choice to the question.
     * 
     * @param choice the answer choice to add.
     */
    public void addAnswerChoice(AnswerChoice choice) {
       this.answerChoices.add(choice);
       this.choicesByAnswerKey.put(choice.getAnswerKey(), choice);
    }
    
    /**
     * Returns the answer choices for the question.
     * 
     * @return the answer choices.
     */
    public Collection<AnswerChoice> getAnswerChoices() {
       return answerChoices;
    }
    
    /**
     * Returns the answer choice with the given value.
     * 
     * @param value the value of the requested answer choice.
     * 
     * @return the answer choice with the given value.
     */
    public AnswerChoice getAnswerChoiceByValue(String value) {
       return (AnswerChoice) choicesByAnswerKey.get(value);
    }
    
    /**
     * {@inheritDoc}
     */
    public String toString() {
        return ToStringBuilder.reflectionToString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }

    /**
     * Determines if the objects are equal by comparing the value of the 
     * question keys.
     * 
     * @return true if the question keys are the same.
     */
    public boolean equals(Object o) {
        
        if (!(o instanceof Question)) return false;
        if (this == o) return true;
        Question qo = (Question) o;
        String key1 = getQuestionKey();
        String key2 = qo.getQuestionKey(); 
        return key1 == null ? key2 == null : key1.equalsIgnoreCase(key2);
    }

    /**
     * Returns a hash code for the property.  Bases the hash code on the 
     * question key strings hash.
     * 
     * @return the hash value.
     */
    public int hashCode() { 
        int result = 17;
        String key = getQuestionKey();
        result = result * 37 + (key == null ? 0 : key.hashCode());
        return result;  
    }
}
