/**
 * 
 */
package org.jharks.workflow.survey.engine.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * A collection containing the answers to a particular question.  The collection
 * contains instances of the <CODE>SurveyAnswer</CODE> class.
 * 
 * @author Michael J. Rizzo
 *
 */
public class QuestionAnswers extends ArrayList<SurveyAnswer> {

    private static final long serialVersionUID = 1L;
    
    /** The question that this collection contains the answers to. */
    private Question question;
    
    /** A set containing the answer values. */
    private Set<Object> values = new HashSet<Object>();
    
    /** A map containing the answers keyed by their corresponding answer choice. */
    private Map<AnswerChoice, SurveyAnswer> selectedAnswerChoices = new HashMap<AnswerChoice, SurveyAnswer>();
    
    /**
     * Default Constructor
     */
    public QuestionAnswers() {
          super();
    }
    
    /**
     * Public constructor.
     * 
     * @param question the question being answered.
     */
    public QuestionAnswers(Question question) {
       super();
       this.question = question;
    }
    
    /**
     * Returns the question that this collection contains the answers to. 
     * 
     * @return the question that this collection contains the answers to. 
     */
    public Question getQuestion() {
       return question;
    }
    
    /**
     * Adds an answer to the set of answers to the question.
     * 
     * @param answer the answer to add.
     */
    public boolean add(SurveyAnswer answer) {
       boolean add = super.add(answer);
       values.add(answer.getValue());
       if (answer.getAnswerChoice() != null) {
          selectedAnswerChoices.put(answer.getAnswerChoice(), answer);
       }
       return add;
    }
    
    /**
     * Returns a <CODE>Set</CODE> containing the selected answer choices.  The
     * <CODE>Set</CODE> contains instances of <CODE>AnswerChoice</CODE>.
     * 
     * @return a <CODE>Set</CODE> containing the selected answer choices.
     */
    public Set<AnswerChoice> getSelectedAnswerChoices() {
       return selectedAnswerChoices.keySet();
    }
    
    /**
     * Returns true if the given answer choice was selected, false otherwise.
     * 
     * @param choice the answer choice to check for.
     * 
     * @return true if the given answer choice was selected, false otherwise.
     */
    public boolean isAnswerChoiceSelected(AnswerChoice choice) {
       return selectedAnswerChoices.containsKey(choice);
    }
    
    /**
     * Returns the answer object corresponding to the given answer choice, if any.
     * 
     * @param choice the choice for which to retrieve the answer object.
     * 
     * @return the answer object corresponding to the given answer choice, if any.
     */
    public SurveyAnswer getAnswerByAnswerChoice(AnswerChoice choice) {
       return (SurveyAnswer) selectedAnswerChoices.get(choice);
    }
    
    /**
     * Returns the values of the answers.
     * 
     * @return the values of the answers.
     */
    public Set<Object> getValues() {
       return values;
    }
    
    /**
     * Returns true if one or more of the answers has the given value.
     * 
     * @param value the value to check for.
     * 
     * @return true if one or more of the answers has the given value.
     */
    public boolean containsValue(String value) {
       return values.contains(value);
    }

    
   
}
