package org.jharks.workflow.survey.engine.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;


/**
 * Contains a group of questions.
 */
public class QuestionGroup extends ArrayList<Question> implements Serializable {

    private static final long serialVersionUID = 1L;
    
    /** The database generated id for the question group. */
    private Long generatedGroupId;
    
    /** The ID for the question group. */
    private String groupId;
    
    /** Indicator whether the question group is a sub group (subquestion group). */
    private boolean subquestionGroup = false;

    /** An internal map containing the questions, keyed by ID. */
    protected Map<Long, Question> questions = new HashMap<Long, Question>();
       
    /**
     * Default public constructor
     */
    public QuestionGroup(Long generatedGroupId) {
      this.generatedGroupId = generatedGroupId;
    }
    
    /**
     * Public constructor.
     * 
     * @param groupId the ID for the question group.
     */
    public QuestionGroup(String groupId) {
       this.groupId = groupId;
    }
    
    /**
     * Returns the group ID.
     * 
     * @return the group ID.
     */
    public String getGroupId() {
       return groupId;
    }
    
    /**
     * Set group id (the group name)
     * 
	 * @param groupId the groupId to set
	 */
	public void setGroupId(String groupId) {
		this.groupId = groupId;
	}

	/**
	 * Return the generated group id.
	 * 
	 * @return the generated group id.
	 */
	public Long getGeneratedGroupId() {
		return generatedGroupId;
	}

	/**
	 * Set the generated group id.
	 * 
	 * @param generatedGroupId the generated group id to set.
	 */
	public void setGeneratedGroupId(Long generatedGroupId) {
		this.generatedGroupId = generatedGroupId;
	}
	
	/**
	 * Returns a boolean indicator whether the question
	 * group is actually a subgroup or not.
	 * 
	 * @return true if the question group is a sub group.
	 */
	public boolean isSubquestionGroup() {
		return subquestionGroup;
	}

	/**
	 * Set the indicator for whether the question group
	 * is a subquestion or not.
	 * 
	 * @param subquestionGroup set whether question group is a subgroup.
	 */
	public void setSubquestionGroup(boolean subquestionGroup) {
		this.subquestionGroup = subquestionGroup;
	}

	/**
     * Adds the given question to the question group.
     * 
     * @param question the question to add.
     */
    public boolean add(Question question) {
       questions.put(new Long(question.getQuestionId()), question);
       return super.add(question);
    }
    
    /**
     * Returns the question with the given ID, if it is in this group.
     * 
     * @param questionId the ID of the question to retrieve.
     * 
     * @return the question with the given ID, if it is in this group.
     */
    public Question getQuestion(long questionId) {
       return (Question) questions.get(new Long(questionId));
    }
    
    /**
     * Adds the collection of questions to the question group.
     * 
     * @param questions the questions to add.
     * 
     * @return true.
     */
    public boolean addAll(List<Question> questions) {
       Iterator<Question> qi = questions.iterator();
       while (qi.hasNext()) {
          add(qi.next());
       }
       return true;
    } 
    
    /**
     * {@inheritDoc}
     */
    public String toString() {
        return ToStringBuilder.reflectionToString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
    
    /**
     * Determines if the objects are equal by comparing the value of the 
     * question group keys.
     * 
     * @return true if the question group keys are the same.
     */
    public boolean equals(Object o) {
      if (!(o instanceof QuestionGroup)) return false;
      if (this == o) return true;
      QuestionGroup questionGroup = (QuestionGroup) o;
      Long id1 = getGeneratedGroupId();
      Long id2 = questionGroup.getGeneratedGroupId();
      boolean idCompare = id1 == null ? id2 == null : id1.equals(id2);
      if (!idCompare) return false;
      
      String groupName1 = getGroupId();
      String groupName2 = questionGroup.getGroupId();
      return groupName1 == null ? groupName2 == null : groupName1.equalsIgnoreCase(groupName2);
    }
    
    /**
     * Returns a hash code for the property.  Bases the hash code on the 
     * question group name (id) and database generated id hash.
     * 
     * @return the hash value.
     */
    public int hashCode() { 
        int result = 17;
        Long generatedId = getGeneratedGroupId();
        result = result * 37 + (generatedId == null ? 0 : generatedId.hashCode());
        String groupId = getGroupId();
        result = result * 37 + (groupId == null ? 0 : groupId.hashCode());
        return result;
    }    
}
