package org.jharks.workflow.survey.engine.model;

import java.util.Date;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;

/**
 * Represents a survey record. 
 * 
 * @author Michael J. Rizzo
 */
public class Survey {

	 /** The unique survey identifier. */
    private Long surveyId;
    
    /** The survey start date. */
    private Date startDate;
    
    /** The survey end date. */
    private Date endDate;
    
    /** The survey's last update date time. */
    private Date updateDate;
    
    /** The user id of the individual creating the survey */
    private Long userId;
    
    /** Indicator of the status of the record (incomplete, finalized). */
    private SurveyStatus status = SurveyStatus.INCOMPLETE;
    
    /** Used to track the subject identifier if present. */
    private Long subjectId;
    
    /** The resource bundle name. */
    private String resourceBundleName;
    
    /** Default Constructor */
    public Survey() {
        startDate = new Date();
    }
    
    /**
     * Constructor that takes previously retrieved surveyId.
     */
    public Survey(long surveyId) {
    	this.surveyId = surveyId;
        startDate = new Date();
    }
    
    /**
     * Sets the start date of the survey.
     * 
     * @param startDate the survey start date.
     */
    public void setStartDate(Date startDate) {
       this.startDate = startDate;
    }

    /**
     * Returns the start date of the survey.
     * 
     * @return startDate the survey start date.
     */
    public Date getStartDate() {
       return startDate;
    }

    /**
     * Sets the end date of the survey.
     * 
     * @param endDate the survey end date.
     */
    public void setEndDate(Date endDate) {
       this.endDate = endDate;
    }

    /**
     * Returns the end date of the survey.
     * 
     * @return endDate the survey end date.
     */
    public Date getEndDate() {
       return endDate;
    }
    
    /**
     * Returns the sruvey unique record identifier.
     * 
     * @return Returns the survey Id.
     */
    public long getSurveyId() {
        return surveyId;
    }
    
    /**
     * Retrieve the user id.
     * @return the user id who created the survey record.
     */
    public Long getUserId() {
    	return userId;
    }

    /**
     * Return the subject associated with survey if present.
     * 
     * @return the subjectId.
     */
    public long getSubjectId() {
        return subjectId;
    }

    /**
     * Set the subject id associated with the survey,
     * 
     * @param subjectId the subjectId to set.
     */
    public void setSubjectId(long subjectId) {
        this.subjectId = subjectId;
    }

    /**
     * Set the survey id.
     * 
     * @param surveyId the surveyId to set.
     */
    public void setSurveyId(long surveyId) {
        this.surveyId = surveyId;
    }
    
    /**
     * {@inheritDoc}
     */
    public String toString() {
        return ToStringBuilder.reflectionToString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }

    /**
     * {@inheritDoc}
     */
    public boolean equals(Object o) {
        
        if (!(o instanceof Survey)) return false;
        if (this == o) return true;
        Survey so = (Survey) o;
        if (so.surveyId == so.surveyId) return true; 
        return false;
    }

    /**
     * {@inheritDoc}
     */
    public int hashCode() {
        int result = 17;
        result = result * 37 + surveyId.hashCode(); 
        result = result * 37 + ((subjectId == null)? 0: subjectId.hashCode());
        return result;
    }

    /**
     * Returns the status of the record.
     * 
     * @return the status of the record.
     */
    public SurveyStatus getStatus() {
        return status;
    }

    /**
     * Sets the status of the record.
     * 
     * @param status the status to set.
     */
    public void setStatus(SurveyStatus status) {
        this.status = status;
    }

    /**
     * Return the last update date time for this record.
     * 
     * @return the updated date time for this record.
     */
    public Date getUpdateDate() {
        return updateDate;
    }

    /**
     * The date to be set as the records last update date time.
     * 
     * @param updateDate the update date to set.
     */
    public void setUpdateDate(Date updateDate) {
        this.updateDate = updateDate;
    }

	/**
	 * Return the resource bundle name.
	 * 
	 * @return the resource bundle name.
	 */
	public String getResourceBundleName() {
		return resourceBundleName;
	}

	/**
	 * Set the resource bundle name.
	 * 
	 * @param resourceBundleName the bundle name to set.
	 */
	public void setResourceBundleName(String resourceBundleName) {
		this.resourceBundleName = resourceBundleName;
	}
    
    
}
