package org.jharks.workflow.survey.engine.service;

import java.sql.Connection;
import java.util.ResourceBundle;

import javax.servlet.http.HttpServletRequest;

import org.jharks.workflow.ResponseSet;
import org.jharks.workflow.SessionHistory;
import org.jharks.workflow.survey.engine.model.GroupAnswers;
import org.jharks.workflow.survey.engine.model.GroupBasedAnswerSet;
import org.jharks.workflow.survey.engine.model.Question;
import org.jharks.workflow.survey.engine.model.QuestionAnswers;
import org.jharks.workflow.survey.engine.model.QuestionGroup;


/**
 * Service manager for retrieval of responses.
 * 
 * @author Michael J. Rizzo
 */
public interface ResponseManager {
    
    /**
     * Retrieves all persisted answers for specified group and groups responses
     * by database configured group names.  This method will return localized
     * answer group keys in the returned object structure.
     * @param surveyId the identifier of the survey record.
     * @return all answers organized by groups.
     */
    public GroupBasedAnswerSet getResponsesSummary(String surveyId);
    
    /**
     * Retrieves all persisted answers for specified group and groups responses
     * by database configured group names.
     * @param surveyId the identifier of the survey record.
     * @param localizeAnswerGroups indicator for whether answer groups should be 
     *        localized within object structure. 
     * @parma bundle the resource bundle to use for the localization.
     * @return all answers organized by groups.
     */
    public GroupBasedAnswerSet getResponsesSummary(String surveyId,
            boolean localizeAnswerGroups, ResourceBundle bundle);
    
    /**
     * Retrieves the answers to the given questions from the request.
     * 
     * @param questions the questions for which to retrieve the answers.
     * @param sequence the sequence used to create the sequence numbers for the answers.
     * @param request the request from which to retrieve the results.
     * 
     * @return an <CODE>ResponseSet</CODE> instance containing the user's responses to the survey.
     */
    public ResponseSet getAnswers(QuestionGroup group, 
            HttpServletRequest request);
    
    /**
     * Retrieves the answers to the given question and any subquestions for the answerchoices.
     * @param question
     * @param request the request from which to retrieve the results.
     * @return an <CODE>AnswerMap</CODE> instance containing the user's responses to the survey.
     */
    public QuestionAnswers getAnswers(Question question, HttpServletRequest request);
    
    /**
     * Retrieves all persisted answers for a specified page group and organizes
     * into a group answers structure.
     * @param pageGroup the page group fro which to retireve answers.
     * @param sesionHistorySnapshot a history snapshot retrieved from memory.
     * @return answers organized in a <code>GroupAnswers</code> instance.
     */
    public GroupAnswers getResponsesForPageGroup(String pageGroup, SessionHistory sessionHistorySnapshot);
    
    /**
     * Retrieves all persisted answers for a specified page group and organizes
     * into a group answers structure.
     * @param pageGroup the page group fro which to retireve answers.
     * @param surveyId the survey identifier.
     * @return answers organized in a <code>GroupAnswers</code> instance.
     */
    public GroupAnswers getResponsesForPageGroup(String pageGroup, Long surveyId);
    
    /**
     * Retrieves all persisted answers for a specified page group and organizes
     * into a group answers structure.
     * @param pageGroup the page group fro which to retireve answers.
     * @param surveyId the survey identifier.
     * @param conn a pre-fetched database connection.
     * @return answers organized in a <code>GroupAnswers</code> instance.
     */
    public GroupAnswers getResponsesForPageGroup(String pageGroup, Long surveyId, Connection conn);
    
}
