package org.jharks.workflow.survey.engine.service.impl;

import java.util.ResourceBundle;

import org.apache.log4j.Logger;
import org.jharks.workflow.survey.engine.cache.WorkFlowResourceBundleCache;
import org.jharks.workflow.survey.engine.cache.WorkFlowResourceBundleCacheKey;
import org.jharks.workflow.survey.engine.dao.WorkFlowResourceBundleDao;
import org.jharks.workflow.survey.engine.gui.taglib.PresentationConstants;
import org.jharks.workflow.survey.engine.service.WorkFlowResourceBundleService;
import org.jharks.workflow.survey.util.CustomDbResourceBundle;

public class WorkFlowResourceBundleServiceImpl implements
		WorkFlowResourceBundleService {

	/** The LOGGER instance for this class. */
	private static final Logger LOGGER = Logger.getLogger(WorkFlowResourceBundleServiceImpl.class); 
	
	/** The default country code that can be overidden by setting properties on the spring bean configuration. */
	private String defaultCountryCode = "US";
	
	/** The default language code that can be overidden by setting properties on the spring bean configuration. */
	private String defaultLanguageCode = "en";
	
	/** The default variant code that can be overidden by setting properties on the spring bean configuration. */
	private String defaultVariantCode = null;
	
	/** The injected dao instance for retrieval of WorkFlowResourceBundle instances. */
	WorkFlowResourceBundleDao workFlowResourceBundleDao;
	
	/** The injected cache for workflow resource bundle cache. */
    WorkFlowResourceBundleCache workFlowResourceBundleCache;
	
    /**
     * {@inheritDoc}
     */
	public ResourceBundle getWorkFlowResourceBundle(String baseName) {
		
		WorkFlowResourceBundleCacheKey key = new WorkFlowResourceBundleCacheKey(baseName,
				this.defaultCountryCode, this.defaultLanguageCode, this.defaultVariantCode);
		
		ResourceBundle bundle = getBundleByKey(key);
		
		return bundle;
	}
	
	/** Injection setters */
	public void setWorkFlowResourceBundleDao(WorkFlowResourceBundleDao workFlowResourceBundleDao) {
		this.workFlowResourceBundleDao = workFlowResourceBundleDao;
	}
	
	public void setWorkFlowResourceBundleCache(WorkFlowResourceBundleCache workFlowResourceBundleCache) {
		this.workFlowResourceBundleCache = workFlowResourceBundleCache;
	}

	/**
	 * {@inheritDoc}
	 */
	public void setDefaultCountryCode(String defaultCountryCode) {
		this.defaultCountryCode = defaultCountryCode;
	}

	/**
	 * {@inheritDoc}
	 */
	public void setDefaultLanguageCode(String defaultLanguageCode) {
		this.defaultLanguageCode = defaultLanguageCode;
	}

	/**
	 * {@inheritDoc}
	 */
	public void setDefaultVariantCode(String defaultVariantCode) {
		this.defaultVariantCode = defaultVariantCode;
	}
	
	/**
	 * Method used by Spring configuration to cache the default jharks
	 * resource bundle upon bean initialization.  Requires method to be
	 * set in <code>init-method</code> parameter of bean configuration.
	 */
	public void cacheDefaultResourceBundle() {
		WorkFlowResourceBundleCacheKey key = createDefaultKey();
		
		getBundleByKey(key);
	}
	
	/**
	 * Create the corresponding key for the default bundle.
	 * @return the cache key.
	 */
	protected WorkFlowResourceBundleCacheKey createDefaultKey() {
		return new WorkFlowResourceBundleCacheKey(
				PresentationConstants.DEFAULT_RESOURCE_BUNDLE_NAME,
				this.defaultCountryCode, 
				this.defaultLanguageCode, 
				this.defaultVariantCode);
	}
	
	/**
	 * Return a resource bundle applicable for the passed key.
	 * @param key the key of the resource bundle to retrieve.
	 * @return the found resource bundle or null. 
	 */
	protected CustomDbResourceBundle getBundleByKey(WorkFlowResourceBundleCacheKey key) {
		
		CustomDbResourceBundle bundle = workFlowResourceBundleCache.getBundleFromCache(key);
		
		if (bundle == null) {
		  bundle = workFlowResourceBundleDao.getWorkFlowResourceBundle(key);
		  if (bundle != null)
			  setParentBundle(bundle);
			  workFlowResourceBundleCache.putBundleInCache(bundle);
		}
		
		return bundle;	
	}
	
	/** 
	 * Set the parent on the passed resource bundle.  Within Jhars this will be the 
	 * default bundle stored in the DB respository.
	 * @param bundle
	 */
	protected void setParentBundle(CustomDbResourceBundle bundle) {
		
		if (bundle == null) return;
		
		WorkFlowResourceBundleCacheKey parentKey = createDefaultKey();
		
		CustomDbResourceBundle parent = workFlowResourceBundleCache.getBundleFromCache(parentKey);
		
		if (LOGGER.isDebugEnabled()) {
			LOGGER.debug("Attempting to set parent logger for bundle: [ " + bundle.getBundleName() + " ]");
		}
		
		if (parent != null) {
			if (LOGGER.isDebugEnabled()) LOGGER.debug("Parent bundle set to bundle: [ " + parent.getBundleName() + " ]");
			bundle.setParentDbBackedBundle(parent);
		} else {
			//retrieve default bundle form datbase and set as parent.
			parent = workFlowResourceBundleDao.getWorkFlowResourceBundle(parentKey);
			workFlowResourceBundleCache.putBundleInCache(parent);
			bundle.setParentDbBackedBundle(parent);
		}
	}
}
