package org.jharks.workflow.survey.engine.web;

import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.jharks.workflow.ResponseSet;
import org.jharks.workflow.survey.engine.model.SurveyAnswer;
import org.jharks.workflow.survey.engine.model.AnswerChoice;
import org.jharks.workflow.survey.engine.model.GroupAnswers;
import org.jharks.workflow.survey.engine.model.Question;
import org.jharks.workflow.survey.engine.model.QuestionAnswers;
import org.jharks.workflow.survey.engine.model.QuestionGroup;


/**
 * Retrieves survey answers from an HTTP request.
 */
public class SurveyAnswerRetrieverRequestImpl implements SurveyAnswerRetriever {
   
   /**
    * {@inheritDoc}
    */
   public ResponseSet getAnswers(QuestionGroup group, 
                                  HttpServletRequest request) {
      
       
      GroupAnswers groupAnswers = getAnswers(group, "", request);
      
      ResponseSet responses = new ResponseSet();
      responses.setQuestionGroup(group.getGroupId());
      Collection<QuestionAnswers> parentQuestionAnswers = groupAnswers.getAnswers();
      addAnswersToResponseSet(responses, parentQuestionAnswers);
      /*
      for (QuestionAnswers answers : qa) {
          responses.addResponses(answers.getQuestion().getQuestionKey(), 
                  answers.getValues()); 
          Set<AnswerChoice> selectedAnswerChoices = answers.getSelectedAnswerChoices();
          for (AnswerChoice answerChoice : selectedAnswerChoices) {
			SurveyAnswer surveyAnswer = answers.getAnswerByAnswerChoice(answerChoice);
			GroupAnswers subAnswers = surveyAnswer.getSubAnswers();
			if (subAnswers != null) {
				responses.addResponses(subAnswers.)
			}
          }
      }
      */
      
      return responses;
   }
   
   /**
    * Method that is called recursively to all answers and subanswers of a question to the response set of 
    * the parent level question group.
    * @param responseSet the response set for the parent level question group.
    * @param questionAnswers the collection of question answers to be processed and added to the response set.
    */
   private void addAnswersToResponseSet(ResponseSet responseSet, Collection<QuestionAnswers> questionAnswers) {
	   for (QuestionAnswers answers : questionAnswers) {
	          responseSet.addResponses(answers.getQuestion().getQuestionKey(), 
	                  answers.getValues()); 
	          Set<AnswerChoice> selectedAnswerChoices = answers.getSelectedAnswerChoices();
	          for (AnswerChoice answerChoice : selectedAnswerChoices) {
				SurveyAnswer surveyAnswer = answers.getAnswerByAnswerChoice(answerChoice);
				GroupAnswers subAnswers = surveyAnswer.getSubAnswers();
				if (subAnswers != null) {
					addAnswersToResponseSet(responseSet, subAnswers.getAnswers());
				}
	          }
	      }
   } 
   
   /**
    * Retrieves the answers to the given questions from the request.
    * 
    * @param questions the questions for which to retrieve the answers.
    * @param prefix the prefix to prepend to the form field names when retrieving answers.
    * @param sequence the sequence used to create the sequence numbers for the answers.
    * @param request the request from which to retrieve the results.
    * 
    * @return an <CODE>GroupAnswers</CODE> instance containing the user's responses to the survey.
    */
   private GroupAnswers getAnswers(QuestionGroup group, String prefix, HttpServletRequest request) {
      
      GroupAnswers answers = new GroupAnswers(group.getGroupId());
      Iterator<Question> qi = group.iterator();
      while (qi.hasNext()) {
         Question question = qi.next();
         QuestionAnswers qa = getAnswers(question, prefix, request);
         
         if (qa != null) {
            answers.add(qa);
         }
      }
      return answers;
   }
   
   /**
    * {@inheritDoc}
    */
   public QuestionAnswers getAnswers(Question question, HttpServletRequest request) {
      return getAnswers(question, "", request);
   }
   
   /**
    * Retrieves the answers to the given question and any subquestions for the answerchoices.
    * 
    * @param question the question for which to retrieve the answers.
    * @param prefix the prefix to prepend to the form field names when retrieving answers.
    * @param sequence the sequence used to create the sequence numbers for the answers.
    * @param request the request from which to retrieve the results.
    * 
    * @return an <CODE>AnswerMap</CODE> instance containing the user's responses to the survey.
    */
   private QuestionAnswers getAnswers(Question question, String prefix, HttpServletRequest request) {
      String questionField = buildFormFieldName(prefix, question);
      
      String[] answerValues = request.getParameterValues(questionField);
      if (answerValues != null && answerValues.length > 0) {
         QuestionAnswers answers = new QuestionAnswers(question);
         
         for (int i = 0; i < answerValues.length; i++) {
            SurveyAnswer answer = new SurveyAnswer(answerValues[i]);
            
            //See if there are any subquestions for the answer.
            AnswerChoice ac = question.getAnswerChoiceByValue(answerValues[i]);
            
            if (ac != null) {
               answer.setAnswerChoice(ac);
               QuestionGroup qg = ac.getSubQuestions();
               if (qg != null && qg.size() > 0) {
                  answer.setSubAnswers(getAnswers(qg, questionField + "a" + ac.getId(), request));
               }
            }
            answers.add(answer);
         }
         return answers;
      }
      return null;
   }
   
   /**
    * Builds the form field name for the given question.
    * 
    * @param prefix the prefix to prepend to the form field names when retrieving answers.
    * @param question the question to build the form field for.
    * 
    * @return the form field name for the question.
    */
   private String buildFormFieldName(String prefix, Question question) {
      if (prefix == null) {
         return question.getQuestionKey();
      } else {
         return prefix + question.getQuestionKey();
      }
   }
}
