package org.jharks.workflow.survey.engine.web;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;

import org.jharks.workflow.survey.engine.model.GroupAnswers;
import org.jharks.workflow.survey.engine.model.GroupAnswersKey;
import org.jharks.workflow.survey.engine.model.GroupBasedAnswerSet;
import org.jharks.workflow.survey.engine.model.Question;
import org.jharks.workflow.survey.engine.model.QuestionAnswers;
import org.jharks.workflow.survey.engine.model.SurveyAnswer;
import org.jharks.workflow.survey.util.ResourceUtilities;

/**
 * Builder class within the web tier that assists in output of a survey full response
 * set to a web based view.  This class focuses on a traditional HTML output.
 * 
 * @author Michael J. Rizzo
 *
 */
public class SurveyHtmlOutputBuilder {

	/**
	 * Transform group based answer set into a formatted html ouptut.
	 * @param answerSet the answer set to transform.
	 * @param bundle the resource bundle to use for localization.
	 * @return the formatted output string.
	 */
	public String transformGroupBasedAnswerSet(GroupBasedAnswerSet answerSet, ResourceBundle bundle) {
		
        Map<GroupAnswersKey, GroupAnswers> groupMap = answerSet.getGroupResponses();
	    
	    Set<GroupAnswersKey> groupKeys = groupMap.keySet();
	    
	    StringBuilder stringBuilder = new StringBuilder();
	    
	    for (GroupAnswersKey groupKey : groupKeys) {
	       GroupAnswers groupAnswers = groupMap.get(groupKey);
	       buildGroupLabel(bundle, stringBuilder, groupAnswers.getLocalizedGroupLabel());
	       
	       processQuestionSet(groupAnswers, bundle, stringBuilder);
		}
		
	    return stringBuilder.toString();
	}
	
	/**
	 * Walk through the set of <code>QuestionAnswers</code> instances.  
	 * @param groupAnswers the answer to a specific group to be processed.
	 * @param bundle the resource bundle to use for localization.
	 * @param stringBuilder the string builder containing the formatted content.
	 */
	private void processQuestionSet(GroupAnswers groupAnswers, ResourceBundle bundle, 
			StringBuilder stringBuilder) {
	   Collection<QuestionAnswers> questionAnswerCollection = groupAnswers.getAnswers();
	   for (QuestionAnswers questionAnswers : questionAnswerCollection) {
		   Question question = questionAnswers.getQuestion();
		   String questionKey = question.getQuestionKey();
		   buildQuestionLabel(bundle, stringBuilder, questionKey);
		   processAnswers(bundle, stringBuilder, questionAnswers);
	   }
	}
	
	/**
	 * Build a label for the question group.
	 * @param bundle the resource bundle to use for localization.
	 * @param stringBuilder the string builder containing the output.
	 * @param groupName the group name for which to build the label.
	 */
	private void buildGroupLabel(ResourceBundle bundle, StringBuilder stringBuilder, String groupName) {
		stringBuilder.append("<div class=\"jharks-summary-group-title\" id=\"")
		             .append(groupName)
		             .append("\">")
		             .append(groupName)
		             .append("</div>");			
	}
	
	/**
	 * Build the question display.
	 * @param bundle the resource bundle to use for localization.
	 * @param stringBuilder the string builder containing the output.
	 * @param questionKey the question key to look up for the question text.
	 */
	private void buildQuestionLabel(ResourceBundle bundle, StringBuilder stringBuilder, String questionKey) {
		stringBuilder.append("<br><span class=\"jharks-question-text\">")
                     .append(ResourceUtilities.getLocalizedResourceEntry(questionKey, bundle))
                     .append("</span><br>");	
	}
	
	/**
	 * Process the passed survey answers.
	 * @param bundle the resource bundle to use for localization.
	 * @param stringBuilder the string builder containing the formatted content.
	 * @param answers the passed survey answers.
	 */
	private void processAnswers(ResourceBundle bundle, StringBuilder stringBuilder, List<SurveyAnswer> answers) {
		
		if (answers == null || answers.size() == 0)
		  stringBuilder.append(ResourceUtilities.getLocalizedResourceEntry("no.response", bundle))
            .append("<br>");
		
		for (SurveyAnswer surveyAnswer : answers) {
			stringBuilder.append("[ " + surveyAnswer.getLocalizedAnswerValue() + " ]<br>");
			GroupAnswers subAnswers = surveyAnswer.getSubAnswers();
			if(subAnswers != null) {
				//process sub QuestionAnswers
				buildSubgroupBlockOpening(bundle, stringBuilder, subAnswers.getAnswerGroupId());
				processQuestionSet(subAnswers, bundle, stringBuilder);
				buildSubgroupBlockClosing(bundle, stringBuilder, subAnswers.getAnswerGroupId());
				
			}
		}
	}
	
	/**
	 * Build the opening tags for a subgroup.
	 * @param bundle the resource bundle to use for localization.
	 * @param stringBuilder the string builder containing the formatted content.
	 * @param answerGroupId the id of the answer group.
	 */
	private void buildSubgroupBlockOpening(ResourceBundle bundle, StringBuilder stringBuilder, Long answerGroupId) {
		stringBuilder.append("<div class=\"jharks-summary-subgroup-block\" id=\"")
          .append(answerGroupId)
          .append("\">");
	}
	
	/**
	 * Build the closing tag for the subgroup.
	 * @param bundle the resource bundle to use for localization.
	 * @param stringBuilder the string builder containing the formatted content.
	 * @param answerGroupId the id of the answer group.
	 */
	private void buildSubgroupBlockClosing(ResourceBundle bundle, StringBuilder stringBuilder, Long answerGroupId) {
		stringBuilder.append("</div>");
	}
	
}
