package org.jharks.workflow.survey.engine.web;

import java.io.StringWriter;
import java.util.Collection;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;
import org.jharks.workflow.survey.engine.exception.SurveyEngineException;
import org.jharks.workflow.survey.engine.model.SurveyAnswer;
import org.jharks.workflow.survey.engine.model.AnswerChoice;
import org.jharks.workflow.survey.engine.model.GroupAnswers;
import org.jharks.workflow.survey.engine.model.Question;
import org.jharks.workflow.survey.engine.model.QuestionAnswers;
import org.w3c.dom.Document;
import org.w3c.dom.Element;



/**
 * Utility class used to write survey questions into an XML string.
 */
public class SurveyXmlBuilder {
   
   /** The log for the class. */
   private static Logger logger = Logger.getLogger(SurveyXmlBuilder.class);
   
   /**
    * Converts a collection of <CODE>Question</CODE> instances into an XML string
    * encapsulating all of the information about those questions.  If 
    * <code>GroupAnswers</code> is not null will also attempt to pre-select 
    * existing answers. 
    * 
    * @param pageName the label for the p[age to be displayed.
    * @param questions the questions to encapsulate as XML.
    * @param groupAnswers existing answers to be pre-selected in display.
    * @return a string containing the question data in XML format.
    */
   public static String buildSurveyXml(String pageName, Collection<Question> questions, 
           GroupAnswers groupAnswers) 
     throws SurveyEngineException {
       
       try {
       DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
       
       DocumentBuilder domBuilder = domFactory.newDocumentBuilder();
       Document document = domBuilder.newDocument();
       
       document.setXmlVersion("1.0");
       
       Element survey = document.createElement("survey");
       document.appendChild(survey);
       survey.setAttribute("page", pageName);
       
       buildQuestionElements(document, survey, questions, groupAnswers);
      
       return serializeDomTreeToString(document);
     } catch (ParserConfigurationException pce) {
         logger.error("Parser configuration exception occurred creating " +
                 "new DOM object for report output", pce);
         throw new SurveyEngineException("Parser configuration " +
                 "excpetion occurred creating new DOM object for " +
                 "report output", pce);
     }  catch (TransformerException te) {
         logger.error("Transformere excpetion occurred transforming " +
                 "new DOM object for survey page output", te);
         throw new SurveyEngineException("Transformere excpetion occurred transforming " +
                 "new DOM object for survey page output", te);
     }
   }
   
   /**
    * Serialize DOM Tree into string.
    * @param document the dom document to serialize.
    * @return the string representation of the DOM instance.
    * @throws TransformerException if error occurs serializing DOM Tree instance.
    */
   protected static String serializeDomTreeToString(Document document) throws TransformerException {
       TransformerFactory transformerFactory = TransformerFactory.newInstance ();
       Transformer transformer = transformerFactory.newTransformer();

       transformer.setOutputProperty(OutputKeys.INDENT,"yes");
       transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");

       StringWriter stringWriter = new StringWriter();
       transformer.transform(new DOMSource(document), new StreamResult(stringWriter));
       
       String serializedResult = stringWriter.toString();
       
       if (logger.isDebugEnabled()) {
    	   logger.debug("SERIALIZED XML FOR SURVEY PAGE = [ " + serializedResult + " ]");
       }
       
       return serializedResult;
   }
   
   /**
    * Builds survey XML for a single question and its answers, including any
    * subquestions and their answers.
    * 
    * @param qa the question and answers.
    * 
    * @return a string containing the question and answer data in XML format.
    */
   public static String buildSurveyXml(QuestionAnswers qa) throws SurveyEngineException {
       try {
           DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
           
           DocumentBuilder domBuilder = domFactory.newDocumentBuilder();
           Document document = domBuilder.newDocument();
           
           document.setXmlVersion("1.0");
           
           Element survey = document.createElement("survey");
           document.appendChild(survey);
           
           buildQuestionElement(document, survey, qa.getQuestion(), qa);
           
           return serializeDomTreeToString(document);
         } catch (ParserConfigurationException pce) {
             logger.error("Parser configuration excpetion occurred creating " +
                     "new DOM object for report output", pce);
             throw new SurveyEngineException("Parser configuration " +
                     "exception occurred creating new DOM object for " +
                     "report output", pce);
         }  catch (TransformerException te) {
             logger.error("Transformere excpetion occurred transforming " +
                     "new DOM object for survey page output", te);
             throw new SurveyEngineException("Transformere excpetion occurred transforming " +
                     "new DOM object for survey page output", te);
         } 
   }

   /**
    * Builds the XML elements for a given question, attaching the question XML elements
    * to the given element.
    * 
    * @param elt the "root" element to attach the question elements to.
    * @param questions the questions to translate into XML.
    */
   private static void buildQuestionElements(Document document, Element elt, Collection<Question> questions) {
      buildQuestionElements(document, elt, questions, null);
   }
   
   /**
    * Builds the question elements for the collection of questions and their answers, held in
    * a <CODE>GroupAnswers</CODE> instance.
    * 
    * @param elt the base element to add the question elements to.
    * @param questions the questions to write to XML.
    * @param answers the answers to the questions.
    */
   private static void buildQuestionElements(Document document, Element elt, Collection<Question> questions, GroupAnswers answers) {
      Iterator<Question> qi = questions.iterator();
      while (qi.hasNext()) {
         Question question = (Question) qi.next();
         QuestionAnswers qa = null;
         if (answers != null) {
            qa = answers.getAnswers(question);
         }
         buildQuestionElement(document, elt, question, qa);
      }
   }
   
   /**
    * Builds a single question element for the given question and its answers.
    * 
    * @param elt the element to add the question element to.
    * @param question the question.
    * @param qa the answers to the questions.
    */
   private static void buildQuestionElement(Document document, Element elt, Question question, QuestionAnswers qa) {
      Element questionElt = document.createElement("question");
      elt.appendChild(questionElt);
      
      //questionElt.setAttribute("id", String.valueOf(question.getQuestionId()));
      questionElt.setAttribute("id", String.valueOf(question.getQuestionKey()));
      questionElt.setAttribute("template", question.getLayoutName());
      
      Element questionTextElt = document.createElement("text");
      questionTextElt.appendChild(document.createTextNode(question.getQuestionKey()));
      questionElt.appendChild(questionTextElt);
      
      if (qa != null && qa.size() > 0) {
         
         Element answersElt = document.createElement("answers"); 
         questionElt.appendChild(answersElt);
         
         Iterator<SurveyAnswer> ai = qa.iterator();
         while (ai.hasNext()) {
            SurveyAnswer answer = ai.next();
            Element answerElt = document.createElement("answer");
            if (answer.getValue() == null) {
              answerElt.appendChild(document.createTextNode(""));
            } else {
              answerElt.appendChild(document.createTextNode(answer.getValue()));	
            }
            answersElt.appendChild(answerElt);
         }
      }
      
      Collection<AnswerChoice> choices = question.getAnswerChoices();
      if (choices != null && choices.size() > 0) {
         
         Element choicesElt = document.createElement("answer-choices");
         questionElt.appendChild(choicesElt);
         Iterator<AnswerChoice> ci = choices.iterator();
         while (ci.hasNext()) {
            AnswerChoice choice = (AnswerChoice) ci.next();
            Element choiceElt = document.createElement("answer-choice");
            choiceElt.setAttribute("id", String.valueOf(choice.getId()));
            String alias = (choice.getAnswerChoiceAliasId() == null)? String.valueOf(choice.getId()) : choice.getAnswerChoiceAliasId();
            choiceElt.setAttribute("alias-id", alias);         	
            
            if (qa != null) {
               if (qa.isAnswerChoiceSelected(choice)) {
                  choiceElt.setAttribute("selected", "true");
               } else {
                  choiceElt.setAttribute("selected", "false");
               }
            }
            
            Element answerChoiceLabel = document.createElement("text");
            answerChoiceLabel.appendChild(document.createTextNode(choice.getAnswerLabel()));
            choiceElt.appendChild(answerChoiceLabel);
            
            Element answerKeyElt = document.createElement("value");
            answerKeyElt.appendChild(document.createTextNode(choice.getAnswerKey()));
            choiceElt.appendChild(answerKeyElt);
            
            Collection<Question> subQuestions = choice.getSubQuestions();
            
            //If we have answers for the questions, we'll need to retrieve the subQuestions...
            GroupAnswers subQuestionAnswers = null;
            if (qa != null) {
               SurveyAnswer answer = qa.getAnswerByAnswerChoice(choice);
               if (answer != null) {
                  subQuestionAnswers = answer.getSubAnswers();
               }
            }

            if (subQuestions != null && subQuestions.size() != 0) {
                  
               Element subQuesElt = document.createElement("subquestions");
               choiceElt.appendChild(subQuesElt);
               buildQuestionElements(document, subQuesElt, subQuestions, subQuestionAnswers);
            }
            
            choicesElt.appendChild(choiceElt);
         }
      }
            
   }
   
}

