package org.jharks.workflow.survey.util;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

public abstract class CommonResourceBundle extends ResourceBundle {
	
	/** Collection of resource bundles */
	private static ArrayList<ResourceBundle> bundles = new ArrayList<ResourceBundle>();
	
	/** Resource bundle base names */
	protected String[] baseName;

	/** Default implementation of this abstract class */
	private static DefResourceBundle instance;
	
	/**
	 * Default constructor that sets basename as empty string.
	 */
	public CommonResourceBundle() {
		this(new String[0]);
	}
	
	/**
	 * Sets the resource bundle base names as an array from a string like:
	 * 
	 * test1,test2 etc or test1 test2 etc
	 */
	protected CommonResourceBundle(String baseName) {
		buildBaseName(baseName, " ,");
	}
	
	/**
	 * Sets the name of the resource bundle base names as an array.
	 * Sets the resource bundle base names as an array
	 */
	protected CommonResourceBundle(String[] baseName) {
		this.baseName = baseName;
	}
	
	/**
	 * Return a Default Resource bundle
	 * @return a resource bundle
	 */
	public static ResourceBundle getBundle() {
		if (instance == null)
			instance = new DefResourceBundle();
		return instance;
	}

	/**
	 * Return an array of all the resource bundle names that have been 
	 * configured. 
	 * @return an array of all resource bundle base names
	 */
	public String[] getBaseName() {
		return baseName;
	}

	/**
	 * Adds a resource bundle to the collection of bundles
	 * @param bundle the ResourceBundle to add
	 */
	public static void addResourceBundle(ResourceBundle bundle) {
		bundles.add(bundle);
	}

	/**
	 * Removes a resource bundle from the collection of bundles
	 * 
	 * @param bundle the ResourceBundle to remove
	 */
	public static void removeResourceBundle(ResourceBundle bundle) {
		bundles.remove(bundle);
	}

	/**
	 * Return the enumeration of the keys.
	 * @return Enumeration of the keys
	 */
	public abstract Enumeration<String> getKeys();
	
	/**
	 * Gets an object for the given key from this resource bundle and null if
	 * 
	 * this resource bundle does not contain an object for the given key
	 */
	protected abstract Object handleGetObject(String key);

	/**
	 * Builds the resource bundle base names as an array from a string like:
	 * 
	 * test1,test2 etc or test1 test2 etc
	 */
	protected void buildBaseName(String base, String delim) {

		String string = null;
		try {
			string = System.getProperty(base);
			if (string == null)
				return;
			StringTokenizer tokenizer = new StringTokenizer(string, delim);

			baseName = new String[tokenizer.countTokens()];

			int i = 0;

			while (tokenizer.hasMoreTokens())
				baseName[i++] = tokenizer.nextToken().trim();

		} catch (Exception e) {
			throw new RuntimeException("Can not resolve base name: " + string);
		}

	}


	/** 
	 * Default implementation
	 */
	static class DefResourceBundle extends CommonResourceBundle {

		public DefResourceBundle(String[] baseName) {
			super(baseName);
		}

		public DefResourceBundle(String baseName) {
			super(baseName);
		}

		public DefResourceBundle() {
			this(new String[0]);
		}

		public Enumeration<String> getKeys(){

	         return new Enumeration<String>(){

	            Enumeration<String> enumeration = null;

	            int i = 0;

	            public boolean hasMoreElements(){
	               boolean b = false;

	               while (enumeration == null || !(b = enumeration.hasMoreElements())){
	                  if (i >= bundles.size()){
	                     enumeration = null;
	                     return b;
	                  }
	                  enumeration = ((ResourceBundle)bundles.get(i++)).getKeys();
	              }
	            return b;
	            }

	            public String nextElement(){
	               if (enumeration == null) throw new NoSuchElementException();
	               return enumeration.nextElement();
	            }
	         };
		}

		protected Object handleGetObject(String key) {
			ResourceBundle rb = null;
			String val = null;

			for (int i = 0; i < bundles.size(); i++) {
				rb = (ResourceBundle) bundles.get(i);
				try {
					val = rb.getString(key);
				} catch (Exception e) {
				}
				if (val != null)
					break;
			}
			return val;
		}
	}
}
