package org.jharks.workflow.survey.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.MessageFormat;
import java.util.Collection;
import java.util.Iterator;

import org.apache.log4j.Logger;

/**
 * Database utility class that provides common database tasks.
 */
public abstract class DatabaseUtilities {

	private static Logger LOGGER = Logger.getLogger(DatabaseUtilities.class);

	/** The pattern used to retrieve the next value from a sequence. */
	private static final String SEQUENCE_PATTERN = "SELECT {0}.NEXTVAL FROM DUAL";
	
	/** SQL query to retrieve last auto generated key from MySQL DB engine. */
	private static final String AUTO_INC_KEY_QUERY = "SELECT LAST_INSERT_ID()";
	
	/**
	 * The MessageFormat used to write the SQL to retieve a value from a
	 * sequence.
	 */
	private static MessageFormat sequenceFormat = new MessageFormat(SEQUENCE_PATTERN);

	/**
	 * Method performs a database object cleanup, closes the result set, statement and connection.
	 * @param conn the database connection to close.
	 * @param rs the result set to close.
	 * @param st the statement to close.
	 */
	public static void cleanup(Connection conn, ResultSet rs, Statement st) {
		DatabaseUtilities.cleanup(rs);
		DatabaseUtilities.cleanup(st);
		DatabaseUtilities.closeConnection(conn);
	}

	/**
	 * Method performs a database object cleanup, closes the result set and statement.
	 * @param rs the result set to close.
	 * @param st the statement to close.
	 */
	public static void cleanup(ResultSet rs, Statement st) {
		DatabaseUtilities.cleanup(rs);
		DatabaseUtilities.cleanup(st);
	}

	/**
	 * Method performs a database object cleanup, closes the result set and connection.
	 * @param conn the database connection to close.
	 * @param rs the result set to close.
	 */
	public static void cleanup(Connection conn, ResultSet rs) {
		DatabaseUtilities.cleanup(rs);
		DatabaseUtilities.closeConnection(conn);
	}

	/**
	 * Method performs a database object cleanup, closes the connection.
	 * @param conn the connection to close.
	 */
	public static void cleanup(Connection conn) {
		DatabaseUtilities.closeConnection(conn);
	}

	/**
	 * Method performs a database object cleanup, closes the statement.
	 * @param st the statement to close.
	 */
	public static void cleanup(Statement st) {
		if (st != null) {
			try {
				st.close();
			} catch (Exception e) {
				LOGGER.warn("", e);
			}
		}
	}

	/**
	 * Method performs a database object cleanup, closes the result set.
	 * @param rs the result set to close.
	 */
	public static void cleanup(ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (Exception e) {
				LOGGER.warn("", e);
			}
		}
	}

	/**
	 * Method performs a database object cleanup, closes the connection.
	 * @param conn the database connection to close.
	 */
	public static void closeConnection(Connection conn) {
		if (conn != null) {
			try {
				conn.close();
			} catch (SQLException se) {
				LOGGER.warn("", se);
			}
		}
	}

	/**
	 * Method to support performing a rollback on a connection.
	 * @param conn the connection on which to perform a roll back.
	 */
	public static void doRollback(Connection conn) {
		try {
			conn.rollback();
		} catch (SQLException se) {
			LOGGER.warn("Exception occurred when attempting rollback on the passed database connection.", se);
		}
	}

	/**
	 * Method to return the next value in the specified sequence.
	 * @param name the name of the database sequence.
	 * @param conn the database connection to use.
	 * @return the next value in the sequence.
	 */
	public static long getNextValueInSequence(String name, Connection conn) throws SQLException {
		ResultSet rs = null;

		Statement stmt = null;
		try {
			String sql = sequenceFormat.format(new String[] { name });
			stmt = conn.createStatement();
			if (LOGGER.isDebugEnabled()) {
				LOGGER.debug("Sequence SQL: " + sql);
			}
			rs = stmt.executeQuery(sql);
			rs.next();
			return rs.getLong("NEXTVAL");
		} finally {
			cleanup(rs, stmt);
		}
	}

	/**
	 * Returns the value of a CLOB field as a <code>String</code> from the
	 * current row of a <code>ResultSet</code>.
	 * @param rs the result set containing the CLOB field.
	 * @param columnName the name of the CLOB field.
	 * @return the value of the CLOB field.
	 * @throws SQLException thrown if there is an error while reading the CLOB.
	 */
	public static String getClobAsString(ResultSet rs, String columnName) throws SQLException {
		StringBuffer text = new StringBuffer();
		Reader r = rs.getCharacterStream(columnName);
		try {
			for (int c = r.read(); c > 0; c = r.read()) {
				text.append((char) c);
			}
		} catch (IOException ioe) {
			LOGGER.error("An error occurred while reading lab text", ioe);
			return null;
		}
		return text.toString();
	}

	/**
	 * Returns the value of a CLOB field as a <code>String</code> from the
	 * current row of a <code>ResultSet</code>.
	 * @param rs the result set containing the CLOB field.
	 * @param columnNum the index of the CLOB field.
	 * @return the value of the CLOB field.
	 * @throws SQLException thrown if there is an error while reading the CLOB.
	 */
	public static String getClobAsString(ResultSet rs, int columnNum) throws SQLException {
		StringBuffer text = new StringBuffer();
		Reader r = rs.getCharacterStream(columnNum);
		try {
			for (int c = r.read(); c > 0; c = r.read()) {
				text.append((char) c);
			}
		} catch (IOException ioe) {
			LOGGER.error("An error occurred while reading lab text", ioe);
			return null;
		}
		return text.toString();
	}
	
	/**
	 * Returns the value of the Clob column in an ascii stream.
	 * @param rs the result set containing the CLOB field.
	 * @param columnNum the index of the CLOB field.
	 * @return the value of the CLOB field.
	 * @throws SQLException thrown if there is an error while reading the CLOB.
	 */
	public static InputStream getClobAsStream(ResultSet rs, int columnNum) throws SQLException {
		return rs.getAsciiStream(columnNum);
	}
	
	/**
	 * Returns the value of the Clob column in an ascii stream.
	 * @param rs the result set containing the CLOB field.
	 * @param columnName the name of the CLOB field.
	 * @return the value of the CLOB field.
	 * @throws SQLException thrown if there is an error while reading the CLOB.
	 */
	public static InputStream getClobAsStream(ResultSet rs, String columnName) throws SQLException {
		return rs.getAsciiStream(columnName);
	}
	
	/**
	 * Returns the last generated key for the specified connection.
	 * @param conn A database connection.
	 * @return the last generated key for the specified connection.
	 * @throws SQLException if error occurs retrieving key.
	 */
	public static Integer getGeneratedKey(Connection conn) throws SQLException {
	    Statement st = null;
	    ResultSet rs = null;
	    Integer key = null;
	       
	    st = conn.createStatement();
	    rs = st.executeQuery(AUTO_INC_KEY_QUERY);
	         
	    if (rs.next()) {
	        key = new Integer( rs.getInt(1) );
	    } else {
	        throw new SQLException("Key not available");
	    }
	    return key;
	   }

	/**
	 * Writes the given collection as a String representing a
	 * set in SQL.  This method assumes that the collection
	 * contains objects which will be stored in the database
	 * with the type VARCHAR, and should therefore be
	 * enclosed in single quotes.
	 *
	 * For instance, a collection containing the
	 * values "value1", "value2", and "value3" would be written
	 * as <CODE>('value1', 'value2', 'value3')</CODE>.
	 *
	 * @param coll the collection to write.
	 *
	 * @return a String representing the collection for a SQL query.
	 */
	public static String writeSqlVarcharSet(Collection<?> coll) {

	   if (coll == null || coll.size() == 0) {
	      return "()";
	   }
  
	   Iterator<?> collIterator = coll.iterator();

	   StringBuffer sql = new StringBuffer("('");
	   sql.append(collIterator.next().toString() + "'");
       while (collIterator.hasNext()) {
         String nextItem = collIterator.next().toString();
         sql.append(",'").append(nextItem).append("'");
       }

	   sql.append(")");

	   return sql.toString();
	}

	/**
	 * Writes the given collection as a String representing a set
	 * in SQL.  This method assumes that the collection contains
	 * objects that will be stored in the database as a type that
	 * does not need to be enclosed in single quotes.
	 *
	 * For instance, a collection containing the
	 * values "value1", "value2", and "value3" would be written
	 * as <CODE>(value1, value2, value3)</CODE>.
	 *
	 * @param coll the collection to write.
	 *
	 * @return a String representing the collection for a SQL query.
	 */
    public static String writeSqlSet(Collection<?> coll) {

      if (coll == null || coll.size() == 0) {
         return "()";
      }

      Iterator<?> collIterator = coll.iterator();

      StringBuilder sql = new StringBuilder("(");
      sql.append(collIterator.next().toString());

      while (collIterator.hasNext()) {
         String nextItem = collIterator.next().toString();
         sql.append(",").append(nextItem);
      }

      sql.append(")");

      return sql.toString();
    }

   /**
    * Writes the given array as a String representing a
    * set in SQL.  This method assumes that the collection
    * contains objects which will be stored in the database
    * with the type VARCHAR, and should therefore be
    * enclosed in single quotes.
    *
    * For instance, an array containing the
    * values "value1", "value2", and "value3" would be written
    * as <CODE>('value1', 'value2', 'value3')</CODE>.
    *
    * @param array the array to write.
    *
    * @return a String representing the array for a SQL query.
    */
   public static String writeSqlVarcharSet(Object[] array) {
      if (array == null || array.length == 0) {
         return "()";
      }

      StringBuilder sql = new StringBuilder("('" + array[0].toString() + "'");

      for (int i = 0; i < array.length; i++) {
        sql.append(",'").append(array[0].toString()).append("'");
      }

      sql.append(")");

      return sql.toString();
   }
	   
   /**
    * Writes the given array as a String representing a set
    * in SQL.  This method assumes that the collection contains
    * objects that will be stored in the database as a type that
    * does not need to be enclosed in single quotes.
    *
    * For instance, an array containing the
    * values "value1", "value2", and "value3" would be written
    * as <CODE>(value1, value2, value3)</CODE>.
    *
    * @param array the array to write.
    *
    * @return a String representing the array for a SQL query.
    */
   public static String writeSqlSet(Object[] array) {

      if (array == null || array.length == 0) {
         return "()";
      }

      StringBuilder sql = new StringBuilder("(" + array[0].toString());

      for (int i = 1; i < array.length; i++) {
         sql.append(",").append(array[i].toString());
      }

      sql.append(")");

      return sql.toString();
   }

   /**
    * Writes the given array as a String representing a set
    * in SQL.
    *
    * For instance, an array containing the
    * values 1, 2, and 3 would be written
    * as <CODE>(1, 2, 3)</CODE>.
    *
    * @param array the array to write.
    *
    * @return a String representing the array for a SQL query.
    */
   public static String writeSqlSet(int[] array) {

      if (array == null || array.length == 0) {
         return "()";
      }

      StringBuilder sql = new StringBuilder("(" + array[0]);

      for (int i = 1; i < array.length; i++) {
         sql.append(",").append(array[i]);
      }

      sql.append(")");

      return sql.toString();
   }

   /**
    * Writes the given array as a String representing a set
    * in SQL.
    *
    * For instance, an array containing the
    * values 1, 2, and 3 would be written
    * as <CODE>(1, 2, 3)</CODE>.
    *
    * @param array the array to write.
    *
    * @return a String representing the array for a SQL query.
    */
   public static String writeSqlSet(long[] array) {

      if (array == null || array.length == 0) {
         return "()";
      }

      StringBuilder sql = new StringBuilder("(" + array[0]);

      for (int i = 1; i < array.length; i++) {
         sql.append(",").append(array[i]);
      }

      sql.append(")");

      return sql.toString();
   }
}
