package org.jharks.workflow.survey.util;

import java.io.InputStream;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;
import org.jharks.workflow.survey.engine.service.WorkFlowResourceBundleService;
import org.jharks.workflow.survey.engine.web.spring.AppContext;
import org.springframework.context.ApplicationContext;

/**
 * Utilities class that provides utility methods for file path operations.
 * @author Michael J. Rizzo
 *
 */
public class FilePathUtilities {

	private static final Logger LOGGER = Logger.getLogger(FilePathUtilities.class);
	
    /**
     * Given a url, converts the passed entity from
     * the existing file extension to a faces url call.  Prepends the 
     * faces servlet mapping and converts the extension to .jsp.  
     * 
     * @param url The url to convert.
     * 
     * @return a new faces url.
     */
    public static String getFacesMapping(String url) {
        
        StringBuffer newPath = new StringBuffer("/app/");
        int lastDot = url.lastIndexOf(".");
        if (lastDot >= 0) {
            newPath.append(url.substring(0, lastDot))
                   .append(".jsp");
        } else {
            newPath.append(url);
        }
        
        return newPath.toString();
    }
    
    /**
     * A helper method to retrieve a resource as a stream.  Abstracted this
     * helper method due to the class loading behavior of JBOSS.  JBOSS seems 
     * to require that the current threads classloader be used to properly find
     * a resource.
     * 
     * @param resourceName The resource to retrieve.
     */
    public static InputStream getResourceStream(String resourceName) {
        
        return Thread.currentThread().getContextClassLoader().getResourceAsStream(resourceName);
        
    }
    
    /**
     * Method that calls appropriate underlying methods based on whehter the survey engine is
     * using file based or database based resources.
     * @param baseName the name of the resource bundle to retrieve.
     * @param databaseBacked indeicates whether to retireve a database backed resource or not. 
     * @return the resource bundle.
     */
    public static ResourceBundle getWorkFlowResourceBundleForXsl(String baseName, String databaseBacked) {
    	if (LOGGER.isDebugEnabled()) {
    		LOGGER.debug("Resource bundle base name passed = [ " + baseName + " ]");
    		LOGGER.debug("DB backed resource bundles mode ON = [ " + databaseBacked + " ]");
    	}
    	if (databaseBacked.equalsIgnoreCase("true")) {
    		return getDbBackedResourceBundle(baseName);
    	} else {
    		Locale locale = new Locale("en");
    		return getBundle(baseName, locale);
    	}
    }
    
    /**
     * Retrieve a <object>ResourceBundle</object> for a specific locale.
     * @param baseName the name of the <object>ResourceBundle</object> to retrieve.
     * @param locale the locale of the bundle to be retrieved.
     * @return the identified <object>ResourceBundle</object>.
     */
    public static ResourceBundle getBundle(String baseName, Locale locale) {
        return ResourceBundle.getBundle(baseName, locale,
                Thread.currentThread().getContextClassLoader());
    }
    
    public static ResourceBundle getDbBackedResourceBundle(String baseName) {
    	ApplicationContext appCtx = AppContext.getApplicationContext();
    	WorkFlowResourceBundleService bundleService = (WorkFlowResourceBundleService) 
    	  appCtx.getBean("org.jharks.workFlowResourceBundleService");
    	
    	ResourceBundle resourceBundle = bundleService.getWorkFlowResourceBundle(baseName);
    	
    	return resourceBundle;
    }
    
    /**
     * A method that is used by the XSL stylesheet to contextually build the client side validation
     * script inclusion.
     * @param bundle the resource bundle on which to check for the question group valdiation key.
     * @param resourceKey the key for the question group (questionGroupName.validation)
     * @return an empty string if the key does not exist or return the object value from the resource 
     *         (the validation path).
     */
    public static String getJharksValidationScriptPath(ResourceBundle bundle, String resourceKey) {
    	String scriptPath = "";
    	try {
    		scriptPath = bundle.getString(resourceKey);
    	} catch (Exception e) {
            // ignore		
    	}
    	return scriptPath;
    }
    
}

