package org.jharks.workflow.survey.util;

import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Iterator;

/**
 * Provides utilities that are frequently used while creating
 * SQL statements.
 * 
 * @author Michael J. Rizzo
 * @version 1.0
 */
public class SqlUtilities {

   /** The date format used to convert dates to Oracle format within Java. */
   private static final String JAVA_DATE_FORMAT = "dd-MMM-yyyy HH:mm:ss";

   /** The format used to convert dates within Oracle. */
   private static final String ORACLE_DATE_FORMAT = "DD-MON-YYYY HH24:MI:SS";

   /** The pattern used to write a TO_DATE function. */
   private static final String TO_DATE_PATTERN = "TO_DATE(''{0}'', ''" 
	   + ORACLE_DATE_FORMAT + "'')";

   /** The MessageFormat used to write a TO_DATE function. */
   private static MessageFormat toDateFormat = 
	   new MessageFormat(TO_DATE_PATTERN);

   /** The SimpleDateFormat object used to format dates within Java. */
   private static SimpleDateFormat sdf = new SimpleDateFormat(JAVA_DATE_FORMAT);
   
   /** A single quote. */
   private static final char SINGLE_QUOTE = '\'';
   
   /** A pair of single quotes. */
   private static final String TWO_QUOTES = "''";

   /**
    * Writes the given collection as a String representing a
    * set in SQL.  This method assumes that the collection
    * contains objects which will be stored in the database
    * with the type VARCHAR, and should therefore be
    * enclosed in single quotes.
    *
    * For instance, a collection containing the
    * values "value1", "value2", and "value3" would be written
    * as <CODE>('value1', 'value2', 'value3')</CODE>.
    *
    * @param coll The collection to write.
    *
    * @return a String representing the collection for a SQL query.
    */
   public static String writeSqlVarcharSet(Collection<Object> coll) {

      if (coll == null || coll.size() == 0) {
         return "()";
      }

      Iterator<Object> collIterator = coll.iterator();

      StringBuffer sql = new StringBuffer("('");
      sql.append(collIterator.next().toString() + "'");

      while (collIterator.hasNext()) {
         String nextItem = collIterator.next().toString();
         sql.append(",'").append(nextItem).append("'");
      }

      sql.append(")");

      return sql.toString();
   }

   /**
    * Writes the given collection as a String representing a set
    * in SQL.  This method assumes that the collection contains
    * objects that will be stored in the database as a type that
    * does not need to be enclosed in single quotes.
    *
    * For instance, a collection containing the
    * values "value1", "value2", and "value3" would be written
    * as <CODE>(value1, value2, value3)</CODE>.
    *
    * @param coll The collection to write.
    *
    * @return a String representing the collection for a SQL query.
    */
   public static String writeSqlSet(Collection<Object> coll) {

      if (coll == null || coll.size() == 0) {
         return "()";
      }

      Iterator<Object> collIterator = coll.iterator();

      StringBuffer sql = new StringBuffer("(");
      sql.append(collIterator.next().toString());

      while (collIterator.hasNext()) {
         String nextItem = collIterator.next().toString();
         sql.append(",").append(nextItem);
      }

      sql.append(")");

      return sql.toString();
   }

   /**
    * Writes the given array as a String representing a
    * set in SQL.  This method assumes that the collection
    * contains objects which will be stored in the database
    * with the type VARCHAR, and should therefore be
    * enclosed in single quotes.
    *
    * For instance, an array containing the
    * values "value1", "value2", and "value3" would be written
    * as <CODE>('value1', 'value2', 'value3')</CODE>.
    *
    * @param array The array to write.
    *
    * @return a String representing the array for a SQL query.
    */
   public static String writeSqlVarcharSet(Object[] array) {

      if (array == null || array.length == 0) {
         return "()";
      }

      StringBuffer sql = new StringBuffer("('" + array[0].toString() + "'");

      for (int i = 0; i < array.length; i++) {
         sql.append(",'").append(array[0].toString()).append("'");
      }

      sql.append(")");

      return sql.toString();
   }
   
   /**
    * Writes the given array as a String representing a set
    * in SQL.  This method assumes that the collection contains
    * objects that will be stored in the database as a type that
    * does not need to be enclosed in single quotes.
    *
    * For instance, an array containing the
    * values "value1", "value2", and "value3" would be written
    * as <CODE>(value1, value2, value3)</CODE>.
    *
    * @param array The array to write.
    *
    * @return a String representing the array for a SQL query.
    */
   public static String writeSqlSet(Object[] array) {

      if (array == null || array.length == 0) {
         return "()";
      }

      StringBuffer sql = new StringBuffer("(" + array[0].toString());

      for (int i = 1; i < array.length; i++) {
         sql.append(",").append(array[i].toString());
      }

      sql.append(")");

      return sql.toString();
   }

   /**
    * Writes the given array as a String representing a set
    * in SQL.
    *
    * For instance, an array containing the
    * values 1, 2, and 3 would be written
    * as <CODE>(1, 2, 3)</CODE>.
    *
    * @param array The array to write.
    *
    * @return a String representing the array for a SQL query.
    */
   public static String writeSqlSet(int[] array) {

      if (array == null || array.length == 0) {
         return "()";
      }

      StringBuffer sql = new StringBuffer("(" + array[0]);

      for (int i = 1; i < array.length; i++) {
         sql.append(",").append(array[i]);
      }

      sql.append(")");

      return sql.toString();
   }

   /**
    * Writes the given array as a String representing a set
    * in SQL.
    *
    * For instance, an array containing the
    * values 1, 2, and 3 would be written
    * as <CODE>(1, 2, 3)</CODE>.
    *
    * @param array The array to write.
    *
    * @return a String representing the array for a SQL query.
    */
   public static String writeSqlSet(long[] array) {

      if (array == null || array.length == 0) {
         return "()";
      }

      StringBuffer sql = new StringBuffer("(" + array[0]);

      for (int i = 1; i < array.length; i++) {
         sql.append(",").append(array[i]);
      }

      sql.append(")");

      return sql.toString();
   }
   
   /**
    * Writes a "TO_DATE" function that can be used in a SQL
    * statement involving dates.
    *
    * @param date The date to format.
    *
    * @return a "TO_DATE" function for the given date.
    */
   public static String writeToDateFunction(java.util.Date date) {
      Object[] params = new Object[] { sdf.format(date) };
      return toDateFormat.format(params);
   }

   /**
    * Replaces all single quotes within the given string with
    * double quotes, so they can be used within a SQL query.
    *
    * For instance, if you wanted to find a user whose last name
    * is O'Brien, the where clause of the SQL query should be
    * something like:
    *
    * <pre>
    * WHERE LAST_NAME='O''Brien'
    * </pre>
    *
    * Use this method to convert "O'Brien" to "O''Brien".
    * 
    * @param string The string to escape.
    */
   public static String escapeQuotes(String string) {
      int firstQuote = string.indexOf(SINGLE_QUOTE);
      if (firstQuote >= 0) {
         StringBuffer result = new StringBuffer();
         int start = 0;
         int next = string.indexOf(SINGLE_QUOTE, start);
         while (next >= 0) {
            result.append(string.substring(start,next) + TWO_QUOTES);
            start = next + 1;
            next = string.indexOf(SINGLE_QUOTE, start);
         }
         result.append(string.substring(start));
         return result.toString();
      }

      return string;
   }


}
